# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from oslo_config import cfg
from oslo_log import log as logging
import six

from sahara.plugins import castellan_utils as key_manager
from sahara.plugins import context
from sahara.plugins import swift_helper as swift
from sahara.plugins import topology_helper as th
from sahara.plugins import utils
from sahara_plugin_vanilla.i18n import _
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config_helper
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import oozie_helper
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import utils as u
from sahara_plugin_vanilla.plugins.vanilla import utils as vu

CONF = cfg.CONF
LOG = logging.getLogger(__name__)

HADOOP_CONF_DIR = '/opt/hadoop/etc/hadoop'
OOZIE_CONF_DIR = '/opt/oozie/conf'
HIVE_CONF_DIR = '/opt/hive/conf'
HADOOP_USER = 'hadoop'
HADOOP_GROUP = 'hadoop'

PORTS_MAP = {
    "namenode": [50070, 9000],
    "secondarynamenode": [50090],
    "resourcemanager": [8088, 8032],
    "historyserver": [19888],
    "datanode": [50010, 50075, 50020],
    "nodemanager": [8042],
    "oozie": [11000],
    "hiveserver": [9999, 10000],
    "spark history server": [18080],
    "zookeeper": [2181, 2888, 3888]
}


def configure_cluster(pctx, cluster):
    LOG.debug("Configuring cluster")
    if (CONF.use_identity_api_v3 and CONF.use_domain_for_proxy_users and
            vu.get_hiveserver(cluster) and
            config_helper.is_swift_enabled(pctx, cluster)):
        cluster = utils.create_proxy_user_for_cluster(cluster)

    instances = utils.get_instances(cluster)
    configure_instances(pctx, instances)
    configure_topology_data(pctx, cluster)
    configure_zookeeper(cluster)
    configure_spark(cluster)


def configure_zookeeper(cluster, instances=None):
    zk_servers = vu.get_zk_servers(cluster)
    if zk_servers:
        zk_conf = config_helper.generate_zk_basic_config(cluster)
        zk_conf += _form_zk_servers_to_quorum(cluster, instances)
        _push_zk_configs_to_nodes(cluster, zk_conf, instances)


def _form_zk_servers_to_quorum(cluster, to_delete_instances=None):
    quorum = []
    instances = map(vu.get_instance_hostname, vu.get_zk_servers(cluster))
    if to_delete_instances:
        delete_instances = map(vu.get_instance_hostname, to_delete_instances)
        reserve_instances = list(set(instances) - set(delete_instances))
        # keep the original order of instances
        reserve_instances.sort(key=instances.index)
    else:
        reserve_instances = instances
    for index, instance in enumerate(reserve_instances):
        quorum.append("server.%s=%s:2888:3888" % (index, instance))
    return '\n'.join(quorum)


def _push_zk_configs_to_nodes(cluster, zk_conf, to_delete_instances=None):
    instances = vu.get_zk_servers(cluster)
    if to_delete_instances:
        for instance in to_delete_instances:
            if instance in instances:
                instances.remove(instance)
    for index, instance in enumerate(instances):
        with instance.remote() as r:
            r.write_file_to('/opt/zookeeper/conf/zoo.cfg', zk_conf,
                            run_as_root=True)
            r.execute_command(
                'sudo su - -c "echo %s > /var/zookeeper/myid" hadoop' % index)


def configure_spark(cluster):
    spark_servers = vu.get_spark_history_server(cluster)
    if spark_servers:
        extra = _extract_spark_configs_to_extra(cluster)
        _push_spark_configs_to_node(cluster, extra)


def _push_spark_configs_to_node(cluster, extra):
    spark_master = vu.get_spark_history_server(cluster)
    if spark_master:
        _push_spark_configs_to_existing_node(spark_master, cluster, extra)
        _push_cleanup_job(spark_master, extra)
        with spark_master.remote() as r:
            r.execute_command('sudo su - -c "mkdir /tmp/spark-events" hadoop')


def _push_spark_configs_to_existing_node(spark_master, cluster, extra):

    sp_home = config_helper.get_spark_home(cluster)
    files = {
        os.path.join(sp_home,
                     'conf/spark-env.sh'): extra['sp_master'],
        os.path.join(
            sp_home,
            'conf/spark-defaults.conf'): extra['sp_defaults']
        }

    with spark_master.remote() as r:
        r.write_files_to(files, run_as_root=True)


def _push_cleanup_job(sp_master, extra):
    with sp_master.remote() as r:
        if extra['job_cleanup']['valid']:
            r.write_file_to('/opt/hadoop/tmp-cleanup.sh',
                            extra['job_cleanup']['script'],
                            run_as_root=True)
            r.execute_command("sudo chmod 755 /opt/hadoop/tmp-cleanup.sh")
            cmd = 'sudo sh -c \'echo "%s" > /etc/cron.d/spark-cleanup\''
            r.execute_command(cmd % extra['job_cleanup']['cron'])
        else:
            r.execute_command("sudo rm -f /opt/hadoop/tmp-cleanup.sh")
            r.execute_command("sudo rm -f /etc/cron.d/spark-cleanup")


def _extract_spark_configs_to_extra(cluster):
    sp_master = utils.get_instance(cluster, "spark history server")

    extra = dict()

    config_master = ''
    if sp_master is not None:
        config_master = config_helper.generate_spark_env_configs(cluster)

    # Any node that might be used to run spark-submit will need
    # these libs for swift integration
    config_defaults = config_helper.generate_spark_executor_classpath(cluster)

    extra['job_cleanup'] = config_helper.generate_job_cleanup_config(cluster)
    extra['sp_master'] = config_master
    extra['sp_defaults'] = config_defaults

    return extra


def configure_instances(pctx, instances):
    if len(instances) == 0:
        return

    utils.add_provisioning_step(
        instances[0].cluster_id, _("Configure instances"), len(instances))

    for instance in instances:
        with context.set_current_instance_id(instance.instance_id):
            _configure_instance(pctx, instance)


@utils.event_wrapper(True)
def _configure_instance(pctx, instance):
    _provisioning_configs(pctx, instance)
    _post_configuration(pctx, instance)


def _provisioning_configs(pctx, instance):
    xmls, env = _generate_configs(pctx, instance)
    _push_xml_configs(instance, xmls)
    _push_env_configs(instance, env)


def _generate_configs(pctx, instance):
    hadoop_xml_confs = _get_hadoop_configs(pctx, instance)
    user_xml_confs, user_env_confs = _get_user_configs(
        pctx, instance.node_group)
    xml_confs = utils.merge_configs(user_xml_confs, hadoop_xml_confs)
    env_confs = utils.merge_configs(pctx['env_confs'], user_env_confs)

    return xml_confs, env_confs


def _get_hadoop_configs(pctx, instance):
    cluster = instance.node_group.cluster
    nn_hostname = vu.get_instance_hostname(vu.get_namenode(cluster))
    dirs = _get_hadoop_dirs(instance)
    confs = {
        'Hadoop': {
            'fs.defaultFS': 'hdfs://%s:9000' % nn_hostname
        },
        'HDFS': {
            'dfs.namenode.name.dir': ','.join(dirs['hadoop_name_dirs']),
            'dfs.datanode.data.dir': ','.join(dirs['hadoop_data_dirs']),
            'dfs.hosts': '%s/dn-include' % HADOOP_CONF_DIR,
            'dfs.hosts.exclude': '%s/dn-exclude' % HADOOP_CONF_DIR
        }
    }

    res_hostname = vu.get_instance_hostname(vu.get_resourcemanager(cluster))
    if res_hostname:
        confs['YARN'] = {
            'yarn.nodemanager.aux-services': 'mapreduce_shuffle',
            'yarn.resourcemanager.hostname': '%s' % res_hostname,
            'yarn.resourcemanager.nodes.include-path': '%s/nm-include' % (
                HADOOP_CONF_DIR),
            'yarn.resourcemanager.nodes.exclude-path': '%s/nm-exclude' % (
                HADOOP_CONF_DIR)
        }
        confs['MapReduce'] = {
            'mapreduce.framework.name': 'yarn'
        }
        hs_hostname = vu.get_instance_hostname(vu.get_historyserver(cluster))
        if hs_hostname:
            confs['MapReduce']['mapreduce.jobhistory.address'] = (
                "%s:10020" % hs_hostname)

    oozie = vu.get_oozie(cluster)
    if oozie:
        hadoop_cfg = {
            'hadoop.proxyuser.hadoop.hosts': '*',
            'hadoop.proxyuser.hadoop.groups': 'hadoop'
        }
        confs['Hadoop'].update(hadoop_cfg)

        oozie_cfg = oozie_helper.get_oozie_required_xml_configs(
            HADOOP_CONF_DIR)
        if config_helper.is_mysql_enabled(pctx, cluster):
            oozie_cfg.update(oozie_helper.get_oozie_mysql_configs(cluster))

        confs['JobFlow'] = oozie_cfg

    if config_helper.is_swift_enabled(pctx, cluster):
        swift_configs = {}
        for config in swift.get_swift_configs():
            swift_configs[config['name']] = config['value']

        confs['Hadoop'].update(swift_configs)

    if config_helper.is_data_locality_enabled(pctx, cluster):
        confs['Hadoop'].update(th.TOPOLOGY_CONFIG)
        confs['Hadoop'].update({"topology.script.file.name":
                                HADOOP_CONF_DIR + "/topology.sh"})

    hive_hostname = vu.get_instance_hostname(vu.get_hiveserver(cluster))
    if hive_hostname:
        hive_pass = u.get_hive_password(cluster)

        hive_cfg = {
            'hive.warehouse.subdir.inherit.perms': True,
            'javax.jdo.option.ConnectionURL':
            'jdbc:derby:;databaseName=/opt/hive/metastore_db;create=true'
        }

        if config_helper.is_mysql_enabled(pctx, cluster):
            hive_cfg.update({
                'javax.jdo.option.ConnectionURL':
                'jdbc:mysql://%s/metastore' % hive_hostname,
                'javax.jdo.option.ConnectionDriverName':
                'com.mysql.jdbc.Driver',
                'javax.jdo.option.ConnectionUserName': 'hive',
                'javax.jdo.option.ConnectionPassword': hive_pass,
                'datanucleus.autoCreateSchema': 'false',
                'datanucleus.fixedDatastore': 'true',
                'hive.metastore.uris': 'thrift://%s:9083' % hive_hostname,
            })

        proxy_configs = cluster.cluster_configs.get('proxy_configs')
        if proxy_configs and config_helper.is_swift_enabled(pctx, cluster):
            hive_cfg.update({
                swift.HADOOP_SWIFT_USERNAME: proxy_configs['proxy_username'],
                swift.HADOOP_SWIFT_PASSWORD: key_manager.get_secret(
                    proxy_configs['proxy_password']),
                swift.HADOOP_SWIFT_TRUST_ID: proxy_configs['proxy_trust_id'],
                swift.HADOOP_SWIFT_DOMAIN_NAME: CONF.proxy_user_domain_name
            })

        confs['Hive'] = hive_cfg

    return confs


def _get_user_configs(pctx, node_group):
    ng_xml_confs, ng_env_confs = _separate_configs(node_group.node_configs,
                                                   pctx['env_confs'])
    cl_xml_confs, cl_env_confs = _separate_configs(
        node_group.cluster.cluster_configs, pctx['env_confs'])

    xml_confs = utils.merge_configs(cl_xml_confs, ng_xml_confs)
    env_confs = utils.merge_configs(cl_env_confs, ng_env_confs)
    return xml_confs, env_confs


def _separate_configs(configs, all_env_configs):
    xml_configs = {}
    env_configs = {}
    for service, params in six.iteritems(configs):
        for param, value in six.iteritems(params):
            if all_env_configs.get(service, {}).get(param):
                if not env_configs.get(service):
                    env_configs[service] = {}
                env_configs[service][param] = value
            else:
                if not xml_configs.get(service):
                    xml_configs[service] = {}
                xml_configs[service][param] = value

    return xml_configs, env_configs


def _generate_xml(configs):
    xml_confs = {}
    for service, confs in six.iteritems(configs):
        xml_confs[service] = utils.create_hadoop_xml(confs)

    return xml_confs


def _push_env_configs(instance, configs):
    nn_heap = configs['HDFS']['NameNode Heap Size']
    snn_heap = configs['HDFS']['SecondaryNameNode Heap Size']
    dn_heap = configs['HDFS']['DataNode Heap Size']
    rm_heap = configs['YARN']['ResourceManager Heap Size']
    nm_heap = configs['YARN']['NodeManager Heap Size']
    hs_heap = configs['MapReduce']['JobHistoryServer Heap Size']

    with instance.remote() as r:
        r.replace_remote_string(
            '%s/hadoop-env.sh' % HADOOP_CONF_DIR,
            'export HADOOP_NAMENODE_OPTS=.*',
            'export HADOOP_NAMENODE_OPTS="-Xmx%dm"' % nn_heap)
        r.replace_remote_string(
            '%s/hadoop-env.sh' % HADOOP_CONF_DIR,
            'export HADOOP_SECONDARYNAMENODE_OPTS=.*',
            'export HADOOP_SECONDARYNAMENODE_OPTS="-Xmx%dm"' % snn_heap)
        r.replace_remote_string(
            '%s/hadoop-env.sh' % HADOOP_CONF_DIR,
            'export HADOOP_DATANODE_OPTS=.*',
            'export HADOOP_DATANODE_OPTS="-Xmx%dm"' % dn_heap)
        r.replace_remote_string(
            '%s/yarn-env.sh' % HADOOP_CONF_DIR,
            '\\#export YARN_RESOURCEMANAGER_HEAPSIZE=.*',
            'export YARN_RESOURCEMANAGER_HEAPSIZE=%d' % rm_heap)
        r.replace_remote_string(
            '%s/yarn-env.sh' % HADOOP_CONF_DIR,
            '\\#export YARN_NODEMANAGER_HEAPSIZE=.*',
            'export YARN_NODEMANAGER_HEAPSIZE=%d' % nm_heap)
        r.replace_remote_string(
            '%s/mapred-env.sh' % HADOOP_CONF_DIR,
            'export HADOOP_JOB_HISTORYSERVER_HEAPSIZE=.*',
            'export HADOOP_JOB_HISTORYSERVER_HEAPSIZE=%d' % hs_heap)


def _push_xml_configs(instance, configs):
    xmls = _generate_xml(configs)
    service_to_conf_map = {
        'Hadoop': '%s/core-site.xml' % HADOOP_CONF_DIR,
        'HDFS': '%s/hdfs-site.xml' % HADOOP_CONF_DIR,
        'YARN': '%s/yarn-site.xml' % HADOOP_CONF_DIR,
        'MapReduce': '%s/mapred-site.xml' % HADOOP_CONF_DIR,
        'JobFlow': '%s/oozie-site.xml' % OOZIE_CONF_DIR,
        'Hive': '%s/hive-site.xml' % HIVE_CONF_DIR
    }
    xml_confs = {}
    for service, confs in six.iteritems(xmls):
        if service not in service_to_conf_map.keys():
            continue

        xml_confs[service_to_conf_map[service]] = confs

    _push_configs_to_instance(instance, xml_confs)


def _push_configs_to_instance(instance, configs):
    LOG.debug("Push configs to instance {instance}".format(
        instance=instance.instance_name))
    with instance.remote() as r:
        for fl, data in six.iteritems(configs):
            r.write_file_to(fl, data, run_as_root=True)


def _post_configuration(pctx, instance):
    dirs = _get_hadoop_dirs(instance)
    args = {
        'hadoop_user': HADOOP_USER,
        'hadoop_group': HADOOP_GROUP,
        'hadoop_conf_dir': HADOOP_CONF_DIR,
        'oozie_conf_dir': OOZIE_CONF_DIR,
        'hadoop_name_dirs': " ".join(dirs['hadoop_name_dirs']),
        'hadoop_data_dirs': " ".join(dirs['hadoop_data_dirs']),
        'hadoop_log_dir': dirs['hadoop_log_dir'],
        'hadoop_secure_dn_log_dir': dirs['hadoop_secure_dn_log_dir'],
        'yarn_log_dir': dirs['yarn_log_dir']
    }
    post_conf_script = utils.get_file_text(
        'plugins/vanilla/hadoop2/resources/post_conf.template',
        'sahara_plugin_vanilla')
    post_conf_script = post_conf_script.format(**args)

    with instance.remote() as r:
        r.write_file_to('/tmp/post_conf.sh', post_conf_script)
        r.execute_command('chmod +x /tmp/post_conf.sh')
        r.execute_command('sudo /tmp/post_conf.sh')

        if config_helper.is_data_locality_enabled(pctx, instance.cluster):
            t_script = HADOOP_CONF_DIR + '/topology.sh'
            r.write_file_to(t_script, utils.get_file_text(
                'plugins/vanilla/hadoop2/resources/topology.sh',
                'sahara_plugin_vanilla'), run_as_root=True)
            r.execute_command('chmod +x ' + t_script, run_as_root=True)


def _get_hadoop_dirs(instance):
    dirs = {}
    storage_paths = instance.storage_paths()
    dirs['hadoop_name_dirs'] = _make_hadoop_paths(
        storage_paths, '/hdfs/namenode')
    dirs['hadoop_data_dirs'] = _make_hadoop_paths(
        storage_paths, '/hdfs/datanode')
    dirs['hadoop_log_dir'] = _make_hadoop_paths(
        storage_paths, '/hadoop/logs')[0]
    dirs['hadoop_secure_dn_log_dir'] = _make_hadoop_paths(
        storage_paths, '/hadoop/logs/secure')[0]
    dirs['yarn_log_dir'] = _make_hadoop_paths(
        storage_paths, '/yarn/logs')[0]

    return dirs


def _make_hadoop_paths(paths, hadoop_dir):
    return [path + hadoop_dir for path in paths]


@utils.event_wrapper(
    True, step=_("Configure topology data"), param=('cluster', 1))
def configure_topology_data(pctx, cluster):
    if config_helper.is_data_locality_enabled(pctx, cluster):
        LOG.warning("Node group awareness is not implemented in YARN yet "
                    "so enable_hypervisor_awareness set to False explicitly")
        tpl_map = th.generate_topology_map(cluster, is_node_awareness=False)
        topology_data = "\n".join(
            [k + " " + v for k, v in tpl_map.items()]) + "\n"
        for ng in cluster.node_groups:
            for i in ng.instances:
                i.remote().write_file_to(HADOOP_CONF_DIR + "/topology.data",
                                         topology_data, run_as_root=True)


def get_open_ports(node_group):
    ports = []
    for key in PORTS_MAP:
        if key in node_group.node_processes:
            ports += PORTS_MAP[key]
    return ports



# Copyright (c) 2014 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc

import six


@six.add_metaclass(abc.ABCMeta)
class AbstractVersionHandler(object):

    @abc.abstractmethod
    def get_node_processes(self):
        return

    @abc.abstractmethod
    def get_plugin_configs(self):
        return

    @abc.abstractmethod
    def configure_cluster(self, cluster):
        return

    @abc.abstractmethod
    def start_cluster(self, cluster):
        return

    @abc.abstractmethod
    def validate(self, cluster):
        return

    @abc.abstractmethod
    def scale_cluster(self, cluster, instances):
        return

    @abc.abstractmethod
    def decommission_nodes(self, cluster, instances):
        return

    @abc.abstractmethod
    def validate_scaling(self, cluster, existing, additional):
        return

    @abc.abstractmethod
    def get_edp_engine(self, cluster, job_type):
        return

    def get_edp_job_types(self):
        return []

    def get_edp_config_hints(self, job_type):
        return {}

    @abc.abstractmethod
    def get_open_ports(self, node_group):
        return

    def on_terminate_cluster(self, cluster):
        pass



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import oozie_helper
from sahara_plugin_vanilla.tests.unit import base


class TestOozieHelper(base.SaharaTestCase):

    def setUp(self):
        super(TestOozieHelper, self).setUp()

    def test_get_oozie_required_xml_configs(self):
        hadoop_conf_dir = '/root'
        configs = {
            'oozie.service.ActionService.executor.ext.classes':
            'org.apache.oozie.action.email.EmailActionExecutor,'
            'org.apache.oozie.action.hadoop.HiveActionExecutor,'
            'org.apache.oozie.action.hadoop.ShellActionExecutor,'
            'org.apache.oozie.action.hadoop.SqoopActionExecutor,'
            'org.apache.oozie.action.hadoop.DistcpActionExecutor',

            'oozie.service.SchemaService.wf.ext.schemas':
            'shell-action-0.1.xsd,shell-action-0.2.xsd,shell-action-0.3.xsd,'
            'email-action-0.1.xsd,hive-action-0.2.xsd,hive-action-0.3.xsd,'
            'hive-action-0.4.xsd,hive-action-0.5.xsd,sqoop-action-0.2.xsd,'
            'sqoop-action-0.3.xsd,sqoop-action-0.4.xsd,ssh-action-0.1.xsd,'
            'ssh-action-0.2.xsd,distcp-action-0.1.xsd,distcp-action-0.2.xsd,'
            'oozie-sla-0.1.xsd,oozie-sla-0.2.xsd',

            'oozie.service.JPAService.create.db.schema': 'false',
            'oozie.service.HadoopAccessorService.hadoop.configurations':
                '*=/root'
        }
        ret = oozie_helper.get_oozie_required_xml_configs(hadoop_conf_dir)
        self.assertEqual(ret, configs)

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.hadoop2.'
                'utils.get_oozie_password')
    def test_get_oozie_mysql_configs(self, get_oozie_password):
        get_oozie_password.return_value = '123'
        configs = {
            'oozie.service.JPAService.jdbc.driver':
            'com.mysql.jdbc.Driver',
            'oozie.service.JPAService.jdbc.url':
            'jdbc:mysql://localhost:3306/oozie',
            'oozie.service.JPAService.jdbc.username': 'oozie',
            'oozie.service.JPAService.jdbc.password': '123'
        }
        cluster = mock.Mock()
        ret = oozie_helper.get_oozie_mysql_configs(cluster)
        get_oozie_password.assert_called_once_with(cluster)
        self.assertEqual(ret, configs)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

from sahara.plugins import conductor
from sahara.plugins import context
from sahara.plugins import swift_helper
from sahara.plugins import utils
from sahara_plugin_vanilla.plugins.vanilla import abstractversionhandler as avm
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config as c
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import keypairs
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import recommendations_utils
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import run_scripts as run
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import scaling as sc
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import starting_scripts
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import utils as u
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import validation as vl
from sahara_plugin_vanilla.plugins.vanilla import utils as vu
from sahara_plugin_vanilla.plugins.vanilla.v2_7_5 import config_helper
from sahara_plugin_vanilla.plugins.vanilla.v2_7_5 import edp_engine


CONF = cfg.CONF


class VersionHandler(avm.AbstractVersionHandler):
    def __init__(self):
        self.pctx = {
            'env_confs': config_helper.get_env_configs(),
            'all_confs': config_helper.get_plugin_configs()
        }

    def get_plugin_configs(self):
        return self.pctx['all_confs']

    def get_node_processes(self):
        return {
            "Hadoop": [],
            "MapReduce": ["historyserver"],
            "HDFS": ["namenode", "datanode", "secondarynamenode"],
            "YARN": ["resourcemanager", "nodemanager"],
            "JobFlow": ["oozie"],
            "Hive": ["hiveserver"],
            "Spark": ["spark history server"],
            "ZooKeeper": ["zookeeper"]
        }

    def validate(self, cluster):
        vl.validate_cluster_creating(self.pctx, cluster)

    def update_infra(self, cluster):
        pass

    def configure_cluster(self, cluster):
        c.configure_cluster(self.pctx, cluster)

    def start_cluster(self, cluster):
        keypairs.provision_keypairs(cluster)

        starting_scripts.start_namenode(cluster)
        starting_scripts.start_secondarynamenode(cluster)
        starting_scripts.start_resourcemanager(cluster)

        run.start_dn_nm_processes(utils.get_instances(cluster))
        run.await_datanodes(cluster)

        starting_scripts.start_historyserver(cluster)
        starting_scripts.start_oozie(self.pctx, cluster)
        starting_scripts.start_hiveserver(self.pctx, cluster)
        starting_scripts.start_zookeeper(cluster)

        swift_helper.install_ssl_certs(utils.get_instances(cluster))

        self._set_cluster_info(cluster)
        starting_scripts.start_spark(cluster)

    def decommission_nodes(self, cluster, instances):
        sc.decommission_nodes(self.pctx, cluster, instances)

    def validate_scaling(self, cluster, existing, additional):
        vl.validate_additional_ng_scaling(cluster, additional)
        vl.validate_existing_ng_scaling(self.pctx, cluster, existing)
        zk_ng = utils.get_node_groups(cluster, "zookeeper")
        if zk_ng:
            vl.validate_zookeeper_node_count(zk_ng, existing, additional)

    def scale_cluster(self, cluster, instances):
        keypairs.provision_keypairs(cluster, instances)
        sc.scale_cluster(self.pctx, cluster, instances)

    def _set_cluster_info(self, cluster):
        nn = vu.get_namenode(cluster)
        rm = vu.get_resourcemanager(cluster)
        hs = vu.get_historyserver(cluster)
        oo = vu.get_oozie(cluster)
        sp = vu.get_spark_history_server(cluster)
        info = {}

        if rm:
            info['YARN'] = {
                'Web UI': 'http://%s:%s' % (rm.get_ip_or_dns_name(), '8088'),
                'ResourceManager': 'http://%s:%s' % (
                    rm.get_ip_or_dns_name(), '8032')
            }

        if nn:
            info['HDFS'] = {
                'Web UI': 'http://%s:%s' % (nn.get_ip_or_dns_name(), '50070'),
                'NameNode': 'hdfs://%s:%s' % (nn.hostname(), '9000')
            }

        if oo:
            info['JobFlow'] = {
                'Oozie': 'http://%s:%s' % (oo.get_ip_or_dns_name(), '11000')
            }

        if hs:
            info['MapReduce JobHistory Server'] = {
                'Web UI': 'http://%s:%s' % (hs.get_ip_or_dns_name(), '19888')
            }

        if sp:
            info['Apache Spark'] = {
                'Spark UI': 'http://%s:%s' % (sp.management_ip, '4040'),
                'Spark History Server UI':
                    'http://%s:%s' % (sp.management_ip, '18080')
            }

        ctx = context.ctx()
        conductor.cluster_update(ctx, cluster, {'info': info})

    def get_edp_engine(self, cluster, job_type):
        if job_type in edp_engine.EdpOozieEngine.get_supported_job_types():
            return edp_engine.EdpOozieEngine(cluster)
        if job_type in edp_engine.EdpSparkEngine.get_supported_job_types():
            return edp_engine.EdpSparkEngine(cluster)

        return None

    def get_edp_job_types(self):
        return (edp_engine.EdpOozieEngine.get_supported_job_types() +
                edp_engine.EdpSparkEngine.get_supported_job_types())

    def get_edp_config_hints(self, job_type):
        return edp_engine.EdpOozieEngine.get_possible_job_config(job_type)

    def on_terminate_cluster(self, cluster):
        u.delete_oozie_password(cluster)
        keypairs.drop_key(cluster)

    def get_open_ports(self, node_group):
        return c.get_open_ports(node_group)

    def recommend_configs(self, cluster, scaling):
        recommendations_utils.recommend_configs(cluster,
                                                self.get_plugin_configs(),
                                                scaling)



# Copyright (c) 2014 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re

from sahara.plugins import utils


class VersionFactory(object):
    versions = None
    modules = None
    initialized = False

    @staticmethod
    def get_instance():
        if not VersionFactory.initialized:
            src_dir = os.path.join(os.path.dirname(__file__), '')
            versions = (
                [name[1:].replace('_', '.')
                 for name in os.listdir(src_dir)
                 if (os.path.isdir(os.path.join(src_dir, name))
                     and re.match(r'^v\d+_\d+_\d+$', name))])
            versions.sort(key=utils.natural_sort_key)
            VersionFactory.versions = versions

            VersionFactory.modules = {}
            for version in VersionFactory.versions:
                module_name = ('sahara_plugin_vanilla.plugins.vanilla.v%s.'
                               'versionhandler' % (version.replace('.', '_')))
                module_class = getattr(
                    __import__(module_name, fromlist=['sahara']),
                    'VersionHandler')
                module = module_class()
                VersionFactory.modules[version] = module

            VersionFactory.initialized = True

        return VersionFactory()

    def get_versions(self):
        return VersionFactory.versions

    def get_version_handler(self, version):
        return VersionFactory.modules[version]



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslotest import base

from sahara.plugins import context
from sahara.plugins import db as db_api
from sahara.plugins import main
from sahara.plugins import utils


class SaharaTestCase(base.BaseTestCase):

    def setUp(self):
        super(SaharaTestCase, self).setUp()
        self.setup_context()
        utils.rpc_setup('all-in-one')

    def setup_context(self, username="test_user", tenant_id="tenant_1",
                      auth_token="test_auth_token", tenant_name='test_tenant',
                      service_catalog=None, **kwargs):
        self.addCleanup(context.set_ctx,
                        context.ctx() if context.has_ctx() else None)

        context.set_ctx(context.PluginsContext(
            username=username, tenant_id=tenant_id,
            auth_token=auth_token, service_catalog=service_catalog or {},
            tenant_name=tenant_name, **kwargs))

    def override_config(self, name, override, group=None):
        main.set_override(name, override, group)
        self.addCleanup(main.clear_override, name, group)


class SaharaWithDbTestCase(SaharaTestCase):
    def setUp(self):
        super(SaharaWithDbTestCase, self).setUp()

        self.override_config('connection', "sqlite://", group='database')
        db_api.setup_db()
        self.addCleanup(db_api.drop_db)



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import edp
from sahara.plugins import utils


def get_possible_hive_config_from(file_name):
    '''Return the possible configs, args, params for a Hive job.'''
    config = {
        'configs': utils.load_hadoop_xml_defaults(file_name,
                                                  'sahara_plugin_vanilla'),
        'params': {}
        }
    return config


def get_possible_mapreduce_config_from(file_name):
    '''Return the possible configs, args, params for a MapReduce job.'''
    config = {
        'configs': get_possible_pig_config_from(file_name).get('configs')
        }
    config['configs'] += edp.get_possible_mapreduce_configs()
    return config


def get_possible_pig_config_from(file_name):
    '''Return the possible configs, args, params for a Pig job.'''
    config = {
        'configs': utils.load_hadoop_xml_defaults(file_name,
                                                  'sahara_plugin_vanilla'),
        'args': [],
        'params': {}
        }
    return config



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import starting_scripts
from sahara_plugin_vanilla.tests.unit import base


class StartingScriptsTest(base.SaharaTestCase):

    plugins_path = 'sahara_plugin_vanilla.plugins.vanilla.'

    def setUp(self):
        super(StartingScriptsTest, self).setUp()
        self.cluster = mock.Mock()

    @mock.patch(plugins_path + 'utils.get_namenode')
    @mock.patch(plugins_path + 'hadoop2.starting_scripts._start_namenode')
    def test_start_namenode(self, _start_namenode, get_namenode):
        namenode = mock.Mock()
        get_namenode.return_value = namenode
        starting_scripts.start_namenode(self.cluster)
        get_namenode.assert_called_once_with(self.cluster)
        _start_namenode.assert_called_once_with(namenode)

    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch(plugins_path + 'hadoop2.run_scripts.start_hadoop_process')
    @mock.patch(plugins_path + 'hadoop2.run_scripts.format_namenode')
    def test__start_namenode(self, format_namenode,
                             start_hadoop_process,
                             check_cluster_exists):
        check_cluster_exists.return_value = None
        nn = mock.Mock()
        starting_scripts._start_namenode(nn)
        format_namenode.assert_called_once_with(nn)
        start_hadoop_process.assert_called_once_with(nn, 'namenode')

    @mock.patch(plugins_path +
                'hadoop2.starting_scripts._start_secondarynamenode')
    @mock.patch(plugins_path + 'utils.get_secondarynamenode')
    def test_start_secondarynamenode(self, get_secondarynamenode,
                                     _start_secondarynamenode):
        get_secondarynamenode.return_value = 0
        starting_scripts.start_secondarynamenode(self.cluster)
        get_secondarynamenode.assert_called_once_with(self.cluster)

        get_secondarynamenode.return_value = 1
        starting_scripts.start_secondarynamenode(self.cluster)
        _start_secondarynamenode.assert_called_once_with(1)
        self.assertEqual(get_secondarynamenode.call_count, 2)

    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch(plugins_path + 'hadoop2.run_scripts.start_hadoop_process')
    def test__start_secondarynamenode(self, start_hadoop_process,
                                      check_cluster_exists):
        check_cluster_exists.return_value = None
        snn = mock.Mock()
        starting_scripts._start_secondarynamenode(snn)
        start_hadoop_process.assert_called_once_with(snn,
                                                     'secondarynamenode')

    @mock.patch(plugins_path +
                'hadoop2.starting_scripts._start_resourcemanager')
    @mock.patch(plugins_path + 'utils.get_resourcemanager')
    def test_start_resourcemanager(self, get_resourcemanager,
                                   _start_resourcemanager):
        get_resourcemanager.return_value = 0
        starting_scripts.start_resourcemanager(self.cluster)
        get_resourcemanager.assert_called_once_with(self.cluster)

        get_resourcemanager.return_value = 1
        starting_scripts.start_resourcemanager(self.cluster)
        _start_resourcemanager.assert_called_once_with(1)
        self.assertEqual(get_resourcemanager.call_count, 2)

    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch(plugins_path + 'hadoop2.run_scripts.start_yarn_process')
    def test__start_resourcemanager(self, start_yarn_process,
                                    check_cluster_exists):
        check_cluster_exists.return_value = None
        snn = mock.Mock()
        starting_scripts._start_resourcemanager(snn)
        start_yarn_process.assert_called_once_with(snn,
                                                   'resourcemanager')

    @mock.patch(plugins_path + 'hadoop2.run_scripts.start_historyserver')
    @mock.patch(plugins_path + 'utils.get_historyserver')
    def test_start_historyserver(self, get_historyserver,
                                 start_historyserver):
        get_historyserver.return_value = 0
        starting_scripts.start_historyserver(self.cluster)
        get_historyserver.assert_called_once_with(self.cluster)

        get_historyserver.return_value = 1
        starting_scripts.start_historyserver(self.cluster)
        start_historyserver.assert_called_once_with(1)
        self.assertEqual(get_historyserver.call_count, 2)

    @mock.patch(plugins_path + 'hadoop2.run_scripts.start_oozie_process')
    @mock.patch(plugins_path + 'utils.get_oozie')
    def test_start_oozie(self, get_oozie, start_oozie_process):
        pctx = mock.Mock()
        get_oozie.return_value = 0
        starting_scripts.start_oozie(pctx, self.cluster)
        get_oozie.assert_called_once_with(self.cluster)

        get_oozie.return_value = 1
        starting_scripts.start_oozie(pctx, self.cluster)
        start_oozie_process.assert_called_once_with(pctx, 1)
        self.assertEqual(get_oozie.call_count, 2)

    @mock.patch(plugins_path +
                'hadoop2.run_scripts.start_hiveserver_process')
    @mock.patch(plugins_path + 'utils.get_hiveserver')
    def test_start_hiveserver(self, get_hiveserver,
                              start_hiveserver_process):
        pctx = mock.Mock()
        get_hiveserver.return_value = 0
        starting_scripts.start_hiveserver(pctx, self.cluster)
        get_hiveserver.assert_called_once_with(self.cluster)

        get_hiveserver.return_value = 1
        starting_scripts.start_hiveserver(pctx, self.cluster)
        start_hiveserver_process.assert_called_once_with(pctx, 1)
        self.assertEqual(get_hiveserver.call_count, 2)

    @mock.patch(plugins_path +
                'hadoop2.run_scripts.start_spark_history_server')
    @mock.patch(plugins_path + 'utils.get_spark_history_server')
    def test_start_spark(self, get_spark_history_server,
                         start_spark_history_server):
        get_spark_history_server.return_value = 0
        starting_scripts.start_spark(self.cluster)
        get_spark_history_server.assert_called_once_with(self.cluster)

        get_spark_history_server.return_value = 1
        starting_scripts.start_spark(self.cluster)
        start_spark_history_server.assert_called_once_with(1)
        self.assertEqual(get_spark_history_server.call_count, 2)



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_config import cfg

from sahara.plugins import exceptions as ex
from sahara.plugins import provisioning as p
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config_helper
from sahara_plugin_vanilla.tests.unit import base


class TestConfigHelper(base.SaharaTestCase):

    plugin_path = 'sahara_plugin_vanilla.plugins.vanilla.hadoop2.'

    def setUp(self):
        super(TestConfigHelper, self).setUp()
        self.pctx = mock.Mock()
        self.applicable_target = mock.Mock()
        self.name = mock.Mock()
        self.cluster = mock.Mock()
        self.CONF = cfg.CONF
        self.CONF.import_opt("enable_data_locality",
                             "sahara.topology.topology_helper")

    def test_init_env_configs(self):
        ENV_CONFS = {
            "YARN": {
                'ResourceManager Heap Size': 1024,
                'NodeManager Heap Size': 1024
            },
            "HDFS": {
                'NameNode Heap Size': 1024,
                'SecondaryNameNode Heap Size': 1024,
                'DataNode Heap Size': 1024
            },
            "MapReduce": {
                'JobHistoryServer Heap Size': 1024
            },
            "JobFlow": {
                'Oozie Heap Size': 1024
            }
        }
        configs = config_helper.init_env_configs(ENV_CONFS)
        for config in configs:
            self.assertIsInstance(config, p.Config)

    def test_init_general_configs(self):
        sample_configs = [config_helper.ENABLE_SWIFT,
                          config_helper.ENABLE_MYSQL,
                          config_helper.DATANODES_STARTUP_TIMEOUT,
                          config_helper.DATANODES_DECOMMISSIONING_TIMEOUT,
                          config_helper.NODEMANAGERS_DECOMMISSIONING_TIMEOUT]
        self.CONF.enable_data_locality = False
        self.assertEqual(config_helper._init_general_configs(), sample_configs)

        sample_configs.append(config_helper.ENABLE_DATA_LOCALITY)
        self.CONF.enable_data_locality = True
        self.assertEqual(config_helper._init_general_configs(), sample_configs)

    def test_get_config_value(self):
        cluster = mock.Mock()
        ng = mock.Mock()
        ng.configuration.return_value = mock.Mock()
        ng.configuration.return_value.get.return_value = mock.Mock()
        cl = 'test'
        ng.configuration.return_value.get.return_value.get.return_value = cl
        cluster.node_groups = [ng]
        cl_param = config_helper.get_config_value('pctx', 'service',
                                                  'name', cluster)
        self.assertEqual(cl, cl_param)

        all_confs = mock.Mock()
        all_confs.applicable_target = 'service'
        all_confs.name = 'name'
        all_confs.default_value = 'default'
        pctx = {'all_confs': [all_confs]}
        value = config_helper.get_config_value(pctx, 'service', 'name')
        self.assertEqual(value, 'default')

        pctx = {'all_confs': []}
        self.assertRaises(ex.PluginNotFoundException,
                          config_helper.get_config_value, pctx, 'service',
                          'name')

    @mock.patch(plugin_path + 'config_helper.get_config_value')
    def test_is_swift_enabled(self, get_config_value):
        target = config_helper.ENABLE_SWIFT.applicable_target
        name = config_helper.ENABLE_SWIFT.name
        config_helper.is_swift_enabled(self.pctx, self.cluster)
        get_config_value.assert_called_once_with(self.pctx, target,
                                                 name, self.cluster)

    @mock.patch(plugin_path + 'config_helper.get_config_value')
    def test_is_mysql_enabled(self, get_config_value):
        target = config_helper.ENABLE_MYSQL.applicable_target
        name = config_helper.ENABLE_MYSQL.name
        config_helper.is_mysql_enabled(self.pctx, self.cluster)
        get_config_value.assert_called_once_with(self.pctx, target,
                                                 name, self.cluster)

    @mock.patch(plugin_path + 'config_helper.get_config_value')
    def test_is_data_locality_enabled(self, get_config_value):
        self.CONF.enable_data_locality = False
        enabled = config_helper.is_data_locality_enabled(self.pctx,
                                                         self.cluster)
        self.assertEqual(enabled, False)

        self.CONF.enable_data_locality = True
        target = config_helper.ENABLE_DATA_LOCALITY.applicable_target
        name = config_helper.ENABLE_DATA_LOCALITY.name
        config_helper.is_data_locality_enabled(self.pctx, self.cluster)
        get_config_value.assert_called_once_with(self.pctx, target,
                                                 name, self.cluster)

    def test_generate_spark_env_configs(self):
        configs = 'HADOOP_CONF_DIR=/opt/hadoop/etc/hadoop\n' \
                  'YARN_CONF_DIR=/opt/hadoop/etc/hadoop'
        ret = config_helper.generate_spark_env_configs(self.cluster)
        self.assertEqual(ret, configs)

    @mock.patch('sahara.plugins.utils.get_config_value_or_default')
    def test_generate_spark_executor_classpath(self,
                                               get_config_value_or_default):
        get_config_value_or_default.return_value = None
        path = 'Executor extra classpath'
        ret = config_helper.generate_spark_executor_classpath(self.cluster)
        get_config_value_or_default.assert_called_once_with('Spark',
                                                            path,
                                                            self.cluster)
        self.assertEqual(ret, '\n')

        get_config_value_or_default.return_value = 'test'
        ret = config_helper.generate_spark_executor_classpath(self.cluster)
        self.assertEqual(ret, 'spark.executor.extraClassPath test')

    @mock.patch('sahara.plugins.utils.get_file_text')
    @mock.patch('sahara.plugins.utils.get_config_value_or_default')
    def test_generate_job_cleanup_config(self,
                                         get_config_value_or_default,
                                         get_file_text):
        cron = 'MINIMUM_CLEANUP_MEGABYTES={minimum_cleanup_megabytes};' + \
               'MINIMUM_CLEANUP_SECONDS={minimum_cleanup_seconds};' + \
               'MAXIMUM_CLEANUP_SECONDS={maximum_cleanup_seconds};'
        script = 'MINIMUM_CLEANUP_MEGABYTES=1;' + \
                 'MINIMUM_CLEANUP_SECONDS=1;' + \
                 'MAXIMUM_CLEANUP_SECONDS=1;'
        job_conf = {'valid': True,
                    'cron': (cron,),
                    'script': script}
        get_file_text.return_value = cron
        get_config_value_or_default.return_value = 1
        ret = config_helper.generate_job_cleanup_config(self.cluster)
        self.assertEqual(get_config_value_or_default.call_count, 3)
        self.assertEqual(get_file_text.call_count, 2)
        self.assertEqual(ret, job_conf)

        job_conf = {'valid': False}
        get_config_value_or_default.return_value = 0
        ret = config_helper.generate_job_cleanup_config(self.cluster)
        self.assertEqual(get_config_value_or_default.call_count, 6)
        self.assertEqual(ret, job_conf)

    @mock.patch('sahara.plugins.utils.get_config_value_or_default')
    def test_get_spark_home(self, get_config_value_or_default):
        get_config_value_or_default.return_value = 1
        self.assertEqual(config_helper.get_spark_home(self.cluster), 1)
        get_config_value_or_default.assert_called_once_with('Spark',
                                                            'Spark home',
                                                            self.cluster)

    @mock.patch('sahara.plugins.utils.get_file_text')
    @mock.patch('sahara.plugins.utils.get_config_value_or_default')
    def test_generate_zk_basic_config(self, get_config_value_or_default,
                                      get_file_text):
        key = ('tickTime={ticktime}\n' +
               'initLimit={initlimit}\n' +
               'syncLimit={synclimit}\n')
        actual = 'tickTime=5\ninitLimit=5\nsyncLimit=5\n'

        get_config_value_or_default.return_value = 5
        get_file_text.return_value = key

        ret = config_helper.generate_zk_basic_config(self.cluster)
        self.assertEqual(get_config_value_or_default.call_count, 3)
        self.assertEqual(ret, actual)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_utils import uuidutils

from sahara.plugins import castellan_utils as castellan
from sahara.plugins import utils as u


def get_namenode(cluster):
    return u.get_instance(cluster, "namenode")


def get_resourcemanager(cluster):
    return u.get_instance(cluster, 'resourcemanager')


def get_nodemanagers(cluster):
    return u.get_instances(cluster, 'nodemanager')


def get_oozie(cluster):
    return u.get_instance(cluster, "oozie")


def get_spark_history_server(cluster):
    return u.get_instance(cluster, "spark history server")


def get_hiveserver(cluster):
    return u.get_instance(cluster, "hiveserver")


def get_datanodes(cluster):
    return u.get_instances(cluster, 'datanode')


def get_secondarynamenode(cluster):
    return u.get_instance(cluster, 'secondarynamenode')


def get_historyserver(cluster):
    return u.get_instance(cluster, 'historyserver')


def get_instance_hostname(instance):
    return instance.hostname() if instance else None


def get_zk_servers(cluster):
    return u.get_instances(cluster, 'zookeeper')


def generate_random_password():
    password = uuidutils.generate_uuid()
    return castellan.store_secret(password)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from oslo_config import cfg
import six

from sahara.plugins import provisioning as p
from sahara.plugins import utils
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config_helper

CONF = cfg.CONF
CONF.import_opt("enable_data_locality", "sahara.topology.topology_helper")

CORE_DEFAULT = utils.load_hadoop_xml_defaults(
    'plugins/vanilla/v2_7_1/resources/core-default.xml',
    'sahara_plugin_vanilla')

HDFS_DEFAULT = utils.load_hadoop_xml_defaults(
    'plugins/vanilla/v2_7_1/resources/hdfs-default.xml',
    'sahara_plugin_vanilla')

MAPRED_DEFAULT = utils.load_hadoop_xml_defaults(
    'plugins/vanilla/v2_7_1/resources/mapred-default.xml',
    'sahara_plugin_vanilla')

YARN_DEFAULT = utils.load_hadoop_xml_defaults(
    'plugins/vanilla/v2_7_1/resources/yarn-default.xml',
    'sahara_plugin_vanilla')

OOZIE_DEFAULT = utils.load_hadoop_xml_defaults(
    'plugins/vanilla/v2_7_1/resources/oozie-default.xml',
    'sahara_plugin_vanilla')

HIVE_DEFAULT = utils.load_hadoop_xml_defaults(
    'plugins/vanilla/v2_7_1/resources/hive-default.xml',
    'sahara_plugin_vanilla')

_default_executor_classpath = ":".join(
    ['/opt/hadoop/share/hadoop/tools/lib/hadoop-openstack-2.7.1.jar'])

SPARK_CONFS = copy.deepcopy(config_helper.SPARK_CONFS)

SPARK_CONFS['Spark']['OPTIONS'].append(
    {
        'name': 'Executor extra classpath',
        'description': 'Value for spark.executor.extraClassPath'
                       ' in spark-defaults.conf'
                       ' (default: %s)' % _default_executor_classpath,
        'default': '%s' % _default_executor_classpath,
        'priority': 2,
    }
)

XML_CONFS = {
    "Hadoop": [CORE_DEFAULT],
    "HDFS": [HDFS_DEFAULT],
    "YARN": [YARN_DEFAULT],
    "MapReduce": [MAPRED_DEFAULT],
    "JobFlow": [OOZIE_DEFAULT],
    "Hive": [HIVE_DEFAULT]
}

ENV_CONFS = {
    "YARN": {
        'ResourceManager Heap Size': 1024,
        'NodeManager Heap Size': 1024
    },
    "HDFS": {
        'NameNode Heap Size': 1024,
        'SecondaryNameNode Heap Size': 1024,
        'DataNode Heap Size': 1024
    },
    "MapReduce": {
        'JobHistoryServer Heap Size': 1024
    },
    "JobFlow": {
        'Oozie Heap Size': 1024
    }
}

# Initialise plugin Hadoop configurations
PLUGIN_XML_CONFIGS = config_helper.init_xml_configs(XML_CONFS)
PLUGIN_ENV_CONFIGS = config_helper.init_env_configs(ENV_CONFS)


def _init_all_configs():
    configs = []
    configs.extend(PLUGIN_XML_CONFIGS)
    configs.extend(PLUGIN_ENV_CONFIGS)
    configs.extend(config_helper.PLUGIN_GENERAL_CONFIGS)
    configs.extend(_get_spark_configs())
    configs.extend(_get_zookeeper_configs())
    return configs


def _get_spark_opt_default(opt_name):
    for opt in SPARK_CONFS["Spark"]["OPTIONS"]:
        if opt_name == opt["name"]:
            return opt["default"]
    return None


def _get_spark_configs():
    spark_configs = []
    for service, config_items in six.iteritems(SPARK_CONFS):
        for item in config_items['OPTIONS']:
            cfg = p.Config(name=item["name"],
                           description=item["description"],
                           default_value=item["default"],
                           applicable_target=service,
                           scope="cluster", is_optional=True,
                           priority=item["priority"])
            spark_configs.append(cfg)
    return spark_configs


def _get_zookeeper_configs():
    zk_configs = []
    for service, config_items in six.iteritems(config_helper.ZOOKEEPER_CONFS):
        for item in config_items['OPTIONS']:
            cfg = p.Config(name=item["name"],
                           description=item["description"],
                           default_value=item["default"],
                           applicable_target=service,
                           scope="cluster", is_optional=True,
                           priority=item["priority"])
            zk_configs.append(cfg)
    return zk_configs


PLUGIN_CONFIGS = _init_all_configs()


def get_plugin_configs():
    return PLUGIN_CONFIGS


def get_xml_configs():
    return PLUGIN_XML_CONFIGS


def get_env_configs():
    return ENV_CONFS



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

from sahara.plugins import exceptions as ex
from sahara.plugins import testutils as tu
from sahara_plugin_vanilla.plugins.vanilla import plugin as p
from sahara_plugin_vanilla.tests.unit import base


class ValidationTest(base.SaharaTestCase):
    def setUp(self):
        super(ValidationTest, self).setUp()
        self.pl = p.VanillaProvider()

    def test_validate(self):
        self.ng = []
        self.ng.append(tu.make_ng_dict("nn", "f1", ["namenode"], 0))
        self.ng.append(tu.make_ng_dict("sn", "f1", ["secondarynamenode"], 0))
        self.ng.append(tu.make_ng_dict("jt", "f1", ["resourcemanager"], 0))
        self.ng.append(tu.make_ng_dict("tt", "f1", ["nodemanager"], 0))
        self.ng.append(tu.make_ng_dict("dn", "f1", ["datanode"], 0))
        self.ng.append(tu.make_ng_dict("hs", "f1", ["historyserver"], 0))
        self.ng.append(tu.make_ng_dict("oo", "f1", ["oozie"], 0))

        self._validate_case(1, 1, 1, 10, 10, 0, 0)
        self._validate_case(1, 0, 1, 1, 4, 0, 0)
        self._validate_case(1, 1, 1, 0, 3, 0, 0)
        self._validate_case(1, 0, 1, 0, 3, 0, 0)
        self._validate_case(1, 1, 0, 0, 3, 0, 0)
        self._validate_case(1, 0, 1, 1, 3, 1, 1)
        self._validate_case(1, 1, 1, 1, 3, 1, 0)

        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(0, 0, 1, 10, 3, 0, 0)
        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(2, 0, 1, 10, 3, 0, 0)

        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(1, 2, 1, 1, 3, 1, 1)

        with testtools.ExpectedException(ex.RequiredServiceMissingException):
            self._validate_case(1, 0, 0, 10, 3, 0, 0)
        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(1, 0, 2, 10, 3, 0, 0)

        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(1, 0, 1, 1, 3, 2, 1)
        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(1, 0, 1, 1, 3, 1, 2)
        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(1, 1, 1, 0, 2, 0, 0)
        with testtools.ExpectedException(ex.RequiredServiceMissingException):
            self._validate_case(1, 0, 1, 1, 3, 0, 1)
        with testtools.ExpectedException(ex.RequiredServiceMissingException):
            self._validate_case(1, 0, 1, 0, 3, 1, 1)
        with testtools.ExpectedException(ex.RequiredServiceMissingException):
            self._validate_case(1, 0, 1, 1, 0, 1, 1)

        cl = self._create_cluster(
            1, 1, 1, 0, 3, 0, 0,
            cluster_configs={'HDFS': {'dfs.replication': 4}})

        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self.pl.validate(cl)

        self.ng.append(tu.make_ng_dict("hi", "f1", ["hiveserver"], 0))
        self.ng.append(tu.make_ng_dict("sh", "f1",
                                       ["spark history server"], 0))

        self._validate_case(1, 1, 0, 0, 3, 0, 0, 1, 0)
        self._validate_case(1, 1, 0, 0, 3, 0, 0, 0, 1)

        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(1, 1, 0, 0, 3, 0, 0, 2, 0)
        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(1, 1, 0, 0, 3, 0, 0, 0, 2)

        self.ng.append(tu.make_ng_dict("zk", "f1", ["zookeeper"], 0))

        self._validate_case(1, 1, 0, 0, 3, 0, 0, 1, 1, 3)

        with testtools.ExpectedException(ex.InvalidComponentCountException):
            self._validate_case(1, 1, 0, 0, 3, 0, 0, 1, 1, 2)

    def _create_cluster(self, *args, **kwargs):
        lst = []
        for i in range(0, len(args)):
            self.ng[i]['count'] = args[i]
            lst.append(self.ng[i])

        return tu.create_cluster("cluster1", "tenant1", "vanilla",
                                 "2.7.1", lst, **kwargs)

    def _validate_case(self, *args):
        cl = self._create_cluster(*args)

        self.pl.validate(cl)



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from sahara.plugins import edp
from sahara.plugins import exceptions as ex
from sahara.plugins import utils as plugin_utils
from sahara_plugin_vanilla.i18n import _
from sahara_plugin_vanilla.plugins.vanilla import confighints_helper
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import edp_engine
from sahara_plugin_vanilla.plugins.vanilla import utils as v_utils


class EdpOozieEngine(edp_engine.EdpOozieEngine):
    @staticmethod
    def get_possible_job_config(job_type):
        if edp.compare_job_type(job_type, edp.JOB_TYPE_HIVE):
            return {
                'job_config': confighints_helper.get_possible_hive_config_from(
                    'plugins/vanilla/v2_8_2/resources/hive-default.xml')}
        if edp.compare_job_type(job_type,
                                edp.JOB_TYPE_MAPREDUCE,
                                edp.JOB_TYPE_MAPREDUCE_STREAMING):
            return {
                'job_config':
                confighints_helper.get_possible_mapreduce_config_from(
                    'plugins/vanilla/v2_8_2/resources/mapred-default.xml')}
        if edp.compare_job_type(job_type, edp.JOB_TYPE_PIG):
            return {
                'job_config': confighints_helper.get_possible_pig_config_from(
                    'plugins/vanilla/v2_8_2/resources/mapred-default.xml')}
        return edp_engine.EdpOozieEngine.get_possible_job_config(job_type)


class EdpSparkEngine(edp.PluginsSparkJobEngine):

    edp_base_version = "2.8.2"

    def __init__(self, cluster):
        super(EdpSparkEngine, self).__init__(cluster)
        self.master = plugin_utils.get_instance(cluster,
                                                "spark history server")
        self.plugin_params["spark-user"] = "sudo -u hadoop "
        self.plugin_params["spark-submit"] = os.path.join(
            plugin_utils.get_config_value_or_default(
                "Spark", "Spark home", self.cluster),
            "bin/spark-submit")
        self.plugin_params["deploy-mode"] = "cluster"
        self.plugin_params["master"] = "yarn"

        driver_cp = plugin_utils.get_config_value_or_default(
            "Spark", "Executor extra classpath", self.cluster)
        self.plugin_params["driver-class-path"] = driver_cp

    @staticmethod
    def edp_supported(version):
        return version >= EdpSparkEngine.edp_base_version

    @staticmethod
    def job_type_supported(job_type):
        return (job_type in
                edp.PluginsSparkJobEngine.get_supported_job_types())

    def validate_job_execution(self, cluster, job, data):
        if (not self.edp_supported(cluster.hadoop_version) or
                not v_utils.get_spark_history_server(cluster)):

            raise ex.PluginInvalidDataException(
                _('Spark {base} or higher required to run {type} jobs').format(
                    base=EdpSparkEngine.edp_base_version, type=job.type))

        super(EdpSparkEngine, self).validate_job_execution(cluster, job, data)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import utils as u


def get_oozie_required_xml_configs(hadoop_conf_dir):
    """Following configs differ from default configs in oozie-default.xml."""
    return {
        'oozie.service.ActionService.executor.ext.classes':
        'org.apache.oozie.action.email.EmailActionExecutor,'
        'org.apache.oozie.action.hadoop.HiveActionExecutor,'
        'org.apache.oozie.action.hadoop.ShellActionExecutor,'
        'org.apache.oozie.action.hadoop.SqoopActionExecutor,'
        'org.apache.oozie.action.hadoop.DistcpActionExecutor',

        'oozie.service.SchemaService.wf.ext.schemas':
        'shell-action-0.1.xsd,shell-action-0.2.xsd,shell-action-0.3.xsd,'
        'email-action-0.1.xsd,hive-action-0.2.xsd,hive-action-0.3.xsd,'
        'hive-action-0.4.xsd,hive-action-0.5.xsd,sqoop-action-0.2.xsd,'
        'sqoop-action-0.3.xsd,sqoop-action-0.4.xsd,ssh-action-0.1.xsd,'
        'ssh-action-0.2.xsd,distcp-action-0.1.xsd,distcp-action-0.2.xsd,'
        'oozie-sla-0.1.xsd,oozie-sla-0.2.xsd',

        'oozie.service.JPAService.create.db.schema': 'false',
        'oozie.service.HadoopAccessorService.hadoop.configurations': '*=%s' % (
            hadoop_conf_dir)
    }


def get_oozie_mysql_configs(cluster):
    return {
        'oozie.service.JPAService.jdbc.driver':
        'com.mysql.jdbc.Driver',
        'oozie.service.JPAService.jdbc.url':
        'jdbc:mysql://localhost:3306/oozie',
        'oozie.service.JPAService.jdbc.username': 'oozie',
        'oozie.service.JPAService.jdbc.password': u.get_oozie_password(
            cluster)
    }



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import exceptions as ex
from sahara.plugins import utils as u
from sahara_plugin_vanilla.i18n import _
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config_helper as cu
from sahara_plugin_vanilla.plugins.vanilla import utils as vu


def validate_cluster_creating(pctx, cluster):
    nn_count = _get_inst_count(cluster, 'namenode')
    if nn_count != 1:
        raise ex.InvalidComponentCountException('namenode', 1, nn_count)

    snn_count = _get_inst_count(cluster, 'secondarynamenode')
    if snn_count > 1:
        raise ex.InvalidComponentCountException('secondarynamenode',
                                                _('0 or 1'), snn_count)

    rm_count = _get_inst_count(cluster, 'resourcemanager')
    if rm_count > 1:
        raise ex.InvalidComponentCountException('resourcemanager', _('0 or 1'),
                                                rm_count)

    hs_count = _get_inst_count(cluster, 'historyserver')
    if hs_count > 1:
        raise ex.InvalidComponentCountException('historyserver', _('0 or 1'),
                                                hs_count)

    nm_count = _get_inst_count(cluster, 'nodemanager')
    if rm_count == 0:
        if nm_count > 0:
            raise ex.RequiredServiceMissingException('resourcemanager',
                                                     required_by='nodemanager')

    oo_count = _get_inst_count(cluster, 'oozie')
    dn_count = _get_inst_count(cluster, 'datanode')
    if oo_count > 1:
        raise ex.InvalidComponentCountException('oozie', _('0 or 1'), oo_count)

    if oo_count == 1:
        if dn_count < 1:
            raise ex.RequiredServiceMissingException('datanode',
                                                     required_by='oozie')

        if nm_count < 1:
            raise ex.RequiredServiceMissingException('nodemanager',
                                                     required_by='oozie')

        if hs_count != 1:
            raise ex.RequiredServiceMissingException('historyserver',
                                                     required_by='oozie')

    spark_hist_count = _get_inst_count(cluster, 'spark history server')
    if spark_hist_count > 1:
        raise ex.InvalidComponentCountException('spark history server',
                                                _('0 or 1'),
                                                spark_hist_count)

    rep_factor = cu.get_config_value(pctx, 'HDFS', 'dfs.replication', cluster)
    if dn_count < rep_factor:
        raise ex.InvalidComponentCountException(
            'datanode', rep_factor, dn_count, _('Number of datanodes must be '
                                                'not less than '
                                                'dfs.replication.'))

    hive_count = _get_inst_count(cluster, 'hiveserver')
    if hive_count > 1:
        raise ex.InvalidComponentCountException('hive', _('0 or 1'),
                                                hive_count)

    zk_count = _get_inst_count(cluster, 'zookeeper')
    if zk_count > 0 and (zk_count % 2) != 1:
        raise ex.InvalidComponentCountException(
            'zookeeper', _('odd'), zk_count, _('Number of zookeeper nodes '
                                               'should be odd.'))


def validate_additional_ng_scaling(cluster, additional):
    rm = vu.get_resourcemanager(cluster)
    scalable_processes = _get_scalable_processes()

    for ng_id in additional:
        ng = u.get_by_id(cluster.node_groups, ng_id)
        if not set(ng.node_processes).issubset(scalable_processes):
            msg = _("Vanilla plugin cannot scale nodegroup with processes: %s")
            raise ex.NodeGroupCannotBeScaled(ng.name,
                                             msg % ' '.join(ng.node_processes))

        if not rm and 'nodemanager' in ng.node_processes:
            msg = _("Vanilla plugin cannot scale node group with processes "
                    "which have no master-processes run in cluster")
            raise ex.NodeGroupCannotBeScaled(ng.name, msg)


def validate_existing_ng_scaling(pctx, cluster, existing):
    scalable_processes = _get_scalable_processes()
    dn_to_delete = 0
    for ng in cluster.node_groups:
        if ng.id in existing:
            if ng.count > existing[ng.id] and "datanode" in ng.node_processes:
                dn_to_delete += ng.count - existing[ng.id]

            if not set(ng.node_processes).issubset(scalable_processes):
                msg = _("Vanilla plugin cannot scale nodegroup "
                        "with processes: %s")
                raise ex.NodeGroupCannotBeScaled(
                    ng.name, msg % ' '.join(ng.node_processes))

    dn_amount = len(vu.get_datanodes(cluster))
    rep_factor = cu.get_config_value(pctx, 'HDFS', 'dfs.replication', cluster)

    if dn_to_delete > 0 and dn_amount - dn_to_delete < rep_factor:
        msg = _("Vanilla plugin cannot shrink cluster because it would be "
                "not enough nodes for replicas (replication factor is %s)")
        raise ex.ClusterCannotBeScaled(
            cluster.name, msg % rep_factor)


def validate_zookeeper_node_count(zk_ng, existing, additional):
    zk_amount = 0
    for ng in zk_ng:
        if ng.id in existing:
            zk_amount += existing[ng.id]
        else:
            zk_amount += ng.count

    for ng_id in additional:
        ng = u.get_by_id(zk_ng, ng_id)
        if "zookeeper" in ng.node_processes:
            zk_amount += ng.count

    if (zk_amount % 2) != 1:
        msg = _("Vanilla plugin cannot scale cluster because it must keep"
                " zookeeper service in odd.")
        raise ex.ClusterCannotBeScaled(zk_ng[0].cluster.name, msg)


def _get_scalable_processes():
    return ['datanode', 'nodemanager', 'zookeeper']


def _get_inst_count(cluster, process):
    return sum([ng.count for ng in u.get_node_groups(cluster, process)])



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from oslo_log import log as logging

from sahara.plugins import context
from sahara.plugins import edp
from sahara.plugins import utils
from sahara_plugin_vanilla.i18n import _
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config_helper
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import oozie_helper
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import utils as u
from sahara_plugin_vanilla.plugins.vanilla import utils as vu

LOG = logging.getLogger(__name__)


def start_dn_nm_processes(instances):
    filternames = ['datanode', 'nodemanager']
    instances = utils.instances_with_services(instances, filternames)

    if len(instances) == 0:
        return

    utils.add_provisioning_step(
        instances[0].cluster_id,
        utils.start_process_event_message("DataNodes, NodeManagers"),
        len(instances))

    with context.PluginsThreadGroup() as tg:
        for instance in instances:
            with context.set_current_instance_id(instance.instance_id):
                processes = set(instance.node_group.node_processes)
                processes = processes.intersection(filternames)
                tg.spawn('vanilla-start-processes-%s' % instance.instance_name,
                         _start_processes, instance, list(processes))


@utils.event_wrapper(True)
def _start_processes(instance, processes):
    with instance.remote() as r:
        if 'datanode' in processes:
            r.execute_command(
                'sudo su - -c "hadoop-daemon.sh start datanode" hadoop')
        if 'nodemanager' in processes:
            r.execute_command(
                'sudo su - -c  "yarn-daemon.sh start nodemanager" hadoop')


def start_hadoop_process(instance, process):
    instance.remote().execute_command(
        'sudo su - -c "hadoop-daemon.sh start %s" hadoop' % process)


def start_yarn_process(instance, process):
    instance.remote().execute_command(
        'sudo su - -c  "yarn-daemon.sh start %s" hadoop' % process)


@utils.event_wrapper(
    True, step=utils.start_process_event_message("HistoryServer"))
def start_historyserver(instance):
    instance.remote().execute_command(
        'sudo su - -c "mr-jobhistory-daemon.sh start historyserver" hadoop')


@utils.event_wrapper(True, step=utils.start_process_event_message("Oozie"))
def start_oozie_process(pctx, instance):
    with context.set_current_instance_id(instance.instance_id):
        with instance.remote() as r:
            if config_helper.is_mysql_enabled(pctx, instance.cluster):
                _start_mysql(r)
                LOG.debug("Creating Oozie DB Schema")
                sql_script = utils.get_file_text(
                    'plugins/vanilla/hadoop2/resources/create_oozie_db.sql',
                    'sahara_plugin_vanilla')

                password = oozie_helper.get_oozie_mysql_configs(
                    instance.cluster)[
                    'oozie.service.JPAService.jdbc.password']
                sql_script = sql_script.replace("password", password)

                script_location = "create_oozie_db.sql"
                r.write_file_to(script_location, sql_script)
                r.execute_command('mysql -u root < %(script_location)s && '
                                  'rm %(script_location)s' %
                                  {"script_location": script_location})

            _oozie_share_lib(r)
            _start_oozie(r)


@utils.event_wrapper(
    True, step=utils.start_process_event_message("Spark History Server"))
def start_spark_history_server(master):
    sp_home = config_helper.get_spark_home(master.cluster)
    with context.set_current_instance_id(master.instance_id):
        with master.remote() as r:
            r.execute_command('sudo su - -c "bash %s" hadoop' % os.path.join(
                sp_home, 'sbin/start-history-server.sh'))


def start_zk_server(instances):
    utils.add_provisioning_step(
        instances[0].cluster_id,
        utils.start_process_event_message("ZooKeeper"),
        len(instances))

    with context.PluginsThreadGroup() as tg:
        for instance in instances:
            with context.set_current_instance_id(instance.instance_id):
                tg.spawn('ZK-start-processes-%s' % instance.instance_name,
                         _start_zk_processes, instance, 'start')


def refresh_zk_servers(cluster, to_delete_instances=None):
    instances = vu.get_zk_servers(cluster)
    if to_delete_instances:
        for instance in to_delete_instances:
            if instance in instances:
                instances.remove(instance)

    utils.add_provisioning_step(
        cluster.id,
        utils.start_process_event_message("ZooKeeper"),
        len(instances))

    with context.PluginsThreadGroup() as tg:
        for instance in instances:
            with context.set_current_instance_id(instance.instance_id):
                tg.spawn('ZK-restart-processes-%s' % instance.instance_name,
                         _start_zk_processes, instance, 'restart')


@utils.event_wrapper(True)
def _start_zk_processes(instance, operation):
    with instance.remote() as r:
        r.execute_command(
            'sudo su - -c "bash /opt/zookeeper/bin/zkServer.sh %s"'
            ' hadoop' % operation)


def format_namenode(instance):
    instance.remote().execute_command(
        'sudo su - -c "hdfs namenode -format" hadoop')


@utils.event_wrapper(
    True,
    step=utils.start_process_event_message("Oozie"), param=('cluster', 0))
def refresh_hadoop_nodes(cluster):
    nn = vu.get_namenode(cluster)
    nn.remote().execute_command(
        'sudo su - -c "hdfs dfsadmin -refreshNodes" hadoop')


@utils.event_wrapper(
    True, step=_("Refresh %s nodes") % "YARN", param=('cluster', 0))
def refresh_yarn_nodes(cluster):
    rm = vu.get_resourcemanager(cluster)
    rm.remote().execute_command(
        'sudo su - -c "yarn rmadmin -refreshNodes" hadoop')


def _oozie_share_lib(remote):
    LOG.debug("Sharing Oozie libs")
    # remote.execute_command('sudo su - -c "/opt/oozie/bin/oozie-setup.sh '
    #                        'sharelib create -fs hdfs://%s:8020" hadoop'
    #                        % nn_hostname)

    # TODO(alazarev) return 'oozie-setup.sh sharelib create' back
    # when #1262023 is resolved

    remote.execute_command(
        'sudo su - -c "mkdir /tmp/oozielib && '
        'tar zxf /opt/oozie/oozie-sharelib-*.tar.gz -C '
        '/tmp/oozielib && '
        'hadoop fs -mkdir /user && '
        'hadoop fs -mkdir /user/hadoop && '
        'hadoop fs -put /tmp/oozielib/share /user/hadoop/ && '
        'rm -rf /tmp/oozielib" hadoop')

    LOG.debug("Creating sqlfile for Oozie")
    remote.execute_command('sudo su - -c "/opt/oozie/bin/ooziedb.sh '
                           'create -sqlfile oozie.sql '
                           '-run Validate DB Connection" hadoop')


def _start_mysql(remote):
    LOG.debug("Starting mysql")
    remote.execute_command('/opt/start-mysql.sh')


def _start_oozie(remote):
    remote.execute_command(
        'sudo su - -c "/opt/oozie/bin/oozied.sh start" hadoop')


@utils.event_wrapper(
    True, step=_("Await %s start up") % "DataNodes", param=('cluster', 0))
def await_datanodes(cluster):
    datanodes_count = len(vu.get_datanodes(cluster))
    if datanodes_count < 1:
        return

    l_message = _("Waiting on %s datanodes to start up") % datanodes_count
    with vu.get_namenode(cluster).remote() as r:
        utils.plugin_option_poll(
            cluster, _check_datanodes_count,
            config_helper.DATANODES_STARTUP_TIMEOUT, l_message, 1, {
                'remote': r, 'count': datanodes_count})


def _check_datanodes_count(remote, count):
    if count < 1:
        return True

    LOG.debug("Checking datanode count")
    exit_code, stdout = remote.execute_command(
        'sudo su -lc "hdfs dfsadmin -report" hadoop | '
        r'grep \'Live datanodes\|Datanodes available:\' | '
        r'grep -o \'[0-9]\+\' | head -n 1')
    LOG.debug("Datanode count='{count}'".format(count=stdout.rstrip()))

    return exit_code == 0 and stdout and int(stdout) == count


def _hive_create_warehouse_dir(remote):
    LOG.debug("Creating Hive warehouse dir")
    remote.execute_command("sudo su - -c 'hadoop fs -mkdir -p "
                           "/user/hive/warehouse' hadoop")


def _hive_copy_shared_conf(remote, dest):
    LOG.debug("Copying shared Hive conf")
    dirname, filename = os.path.split(dest)
    remote.execute_command(
        "sudo su - -c 'hadoop fs -mkdir -p %s && "
        "hadoop fs -put /opt/hive/conf/hive-site.xml "
        "%s' hadoop" % (dirname, dest))


def _hive_create_db(remote):
    LOG.debug("Creating Hive metastore db")
    remote.execute_command("mysql -u root < /tmp/create_hive_db.sql")


def _hive_metastore_start(remote):
    LOG.debug("Starting Hive Metastore Server")
    remote.execute_command("sudo su - -c 'nohup /opt/hive/bin/hive"
                           " --service metastore > /dev/null &' hadoop")


@utils.event_wrapper(
    True, step=utils.start_process_event_message("HiveServer"))
def start_hiveserver_process(pctx, instance):
    with context.set_current_instance_id(instance.instance_id):
        with instance.remote() as r:
            _hive_create_warehouse_dir(r)
            _hive_copy_shared_conf(
                r, edp.get_hive_shared_conf_path('hadoop'))

            if config_helper.is_mysql_enabled(pctx, instance.cluster):
                oozie = vu.get_oozie(instance.node_group.cluster)
                if not oozie or instance.hostname() != oozie.hostname():
                    _start_mysql(r)

                version = instance.cluster.hadoop_version
                sql_script = utils.get_file_text(
                    'plugins/vanilla/v{}/resources/create_hive_db.sql'.format(
                        version.replace('.', '_')), 'sahara_plugin_vanilla')

                sql_script = sql_script.replace(
                    '{{password}}', u.get_hive_password(instance.cluster))
                r.write_file_to('/tmp/create_hive_db.sql', sql_script)
                _hive_create_db(r)
                _hive_metastore_start(r)
                LOG.info("Hive Metastore server at {host} has been "
                         "started".format(host=instance.hostname()))



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import base as pb
from sahara.plugins import exceptions as ex
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import edp_engine
from sahara_plugin_vanilla.tests.unit import base as sahara_base


class EdpOozieEngineTest(sahara_base.SaharaTestCase):

    engine_path = 'sahara.plugins.edp.'

    def setUp(self):
        super(EdpOozieEngineTest, self).setUp()
        self.cluster = mock.Mock()
        self.override_config("plugins", ["vanilla"])
        pb.setup_plugins()
        with mock.patch('sahara.plugins.edp.get_plugin',
                        return_value='test_plugins'):
            self.engine = edp_engine.EdpOozieEngine(self.cluster)

    def test_get_hdfs_user(self):
        self.assertEqual(self.engine.get_hdfs_user(), 'hadoop')

    def test_get_name_node_uri(self):
        cluster = {'info': {
            'HDFS': {
                'NameNode': 'test_url'}}}
        ret = self.engine.get_name_node_uri(cluster)
        self.assertEqual(ret, 'test_url')

    def test_get_oozie_server_uri(self):
        cluster = {'info': {
            'JobFlow': {
                'Oozie': 'test_url'}}}
        ret = self.engine.get_oozie_server_uri(cluster)
        self.assertEqual(ret, 'test_url/oozie/')

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.get_oozie')
    def test_get_oozie_server(self, get_oozie):
        get_oozie.return_value = 'bingo'
        ret = self.engine.get_oozie_server(self.cluster)
        get_oozie.assert_called_once_with(self.cluster)
        self.assertEqual(ret, 'bingo')

    @mock.patch(engine_path + 'PluginsOozieJobEngine.validate_job_execution')
    @mock.patch('sahara.plugins.utils.get_instances_count')
    def test_validate_job_execution(self,
                                    get_instances_count,
                                    validate_job_execution):
        job = mock.Mock()
        data = mock.Mock()
        get_instances_count.return_value = 0
        self.assertRaises(ex.InvalidComponentCountException,
                          self.engine.validate_job_execution,
                          self.cluster, job, data)

        get_instances_count.return_value = 1
        self.engine.validate_job_execution(self.cluster, job, data)
        validate_job_execution.assert_called_once_with(self.cluster,
                                                       job, data)

    @mock.patch('sahara.plugins.edp.create_dir_hadoop2')
    def test_create_hdfs_dir(self, create_dir_hadoop2):
        self.engine.get_hdfs_user = mock.Mock(return_value='test_user')
        remote = mock.Mock()
        dir_name = mock.Mock()
        self.engine.create_hdfs_dir(remote, dir_name)
        create_dir_hadoop2.assert_called_once_with(remote, dir_name,
                                                   'test_user')

    def test_get_resource_manager_uri(self):
        cluster = {'info': {
            'YARN': {
                'ResourceManager': 'test_url'}}}
        ret = self.engine.get_resource_manager_uri(cluster)
        self.assertEqual(ret, 'test_url')



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import testutils as tu
from sahara_plugin_vanilla.plugins.vanilla import plugin as p
from sahara_plugin_vanilla.plugins.vanilla import utils as u
from sahara_plugin_vanilla.tests.unit import base


class TestUtils(base.SaharaWithDbTestCase):

    def setUp(self):
        super(TestUtils, self).setUp()
        self.plugin = p.VanillaProvider()

        self.ng_manager = tu.make_ng_dict(
            'mng', 'f1', ['manager'], 1,
            [tu.make_inst_dict('mng1', 'manager')])
        self.ng_namenode = tu.make_ng_dict(
            'nn', 'f1', ['namenode'], 1,
            [tu.make_inst_dict('nn1', 'namenode')])
        self.ng_resourcemanager = tu.make_ng_dict(
            'jt', 'f1', ['resourcemanager'], 1,
            [tu.make_inst_dict('jt1', 'resourcemanager')])
        self.ng_datanode = tu.make_ng_dict(
            'dn', 'f1', ['datanode'], 2,
            [tu.make_inst_dict('dn1', 'datanode-1'),
             tu.make_inst_dict('dn2', 'datanode-2')])
        self.ng_nodemanager = tu.make_ng_dict(
            'tt', 'f1', ['nodemanager'], 2,
            [tu.make_inst_dict('tt1', 'nodemanager-1'),
             tu.make_inst_dict('tt2', 'nodemanager-2')])
        self.ng_oozie = tu.make_ng_dict(
            'ooz1', 'f1', ['oozie'], 1,
            [tu.make_inst_dict('ooz1', 'oozie')])
        self.ng_hiveserver = tu.make_ng_dict(
            'hs', 'f1', ['hiveserver'], 1,
            [tu.make_inst_dict('hs1', 'hiveserver')])
        self.ng_secondarynamenode = tu.make_ng_dict(
            'snn', 'f1', ['secondarynamenode'], 1,
            [tu.make_inst_dict('snn1', 'secondarynamenode')])

    def test_get_namenode(self):
        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager, self.ng_namenode])
        self.assertEqual('nn1', u.get_namenode(cl).instance_id)

        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager])
        self.assertIsNone(u.get_namenode(cl))

    def test_get_nodemanagers(self):
        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager, self.ng_nodemanager])
        nodemanagers = u.get_nodemanagers(cl)
        self.assertEqual(2, len(nodemanagers))
        self.assertEqual(set(['tt1', 'tt2']),
                         set([nodemanagers[0].instance_id,
                             nodemanagers[1].instance_id]))
        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_namenode])
        self.assertEqual([], u.get_nodemanagers(cl))

    def test_get_oozie(self):
        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager, self.ng_oozie])
        self.assertEqual('ooz1', u.get_oozie(cl).instance_id)

        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager])
        self.assertIsNone(u.get_oozie(cl))

    def test_get_hiveserver(self):
        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager, self.ng_hiveserver])
        self.assertEqual('hs1', u.get_hiveserver(cl).instance_id)

        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager])
        self.assertIsNone(u.get_hiveserver(cl))

    def test_get_datanodes(self):
        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager, self.ng_namenode,
                                self.ng_datanode])
        datanodes = u.get_datanodes(cl)
        self.assertEqual(2, len(datanodes))
        self.assertEqual(set(['dn1', 'dn2']),
                         set([datanodes[0].instance_id,
                              datanodes[1].instance_id]))

        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager])
        self.assertEqual([], u.get_datanodes(cl))

    def test_get_secondarynamenodes(self):
        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager, self.ng_namenode,
                                self.ng_secondarynamenode])
        self.assertEqual('snn1', u.get_secondarynamenode(cl).instance_id)

        cl = tu.create_cluster('cl1', 't1', 'vanilla', '2.7.1',
                               [self.ng_manager])
        self.assertIsNone(u.get_secondarynamenode(cl))



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import edp
from sahara_plugin_vanilla.plugins.vanilla.v2_7_5 import edp_engine
from sahara_plugin_vanilla.tests.unit import base as sahara_base


class Vanilla2ConfigHintsTest(sahara_base.SaharaTestCase):
    @mock.patch(
        'sahara_plugin_vanilla.plugins.vanilla.confighints_helper.'
        'get_possible_hive_config_from',
        return_value={})
    def test_get_possible_job_config_hive(
            self, get_possible_hive_config_from):
        expected_config = {'job_config': {}}
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_HIVE)
        get_possible_hive_config_from.assert_called_once_with(
            'plugins/vanilla/v2_7_5/resources/hive-default.xml')
        self.assertEqual(expected_config, actual_config)

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.hadoop2.edp_engine.'
                'EdpOozieEngine')
    def test_get_possible_job_config_java(self, BaseVanillaEdpOozieEngine):
        expected_config = {'job_config': {}}
        BaseVanillaEdpOozieEngine.get_possible_job_config.return_value = (
            expected_config)
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_JAVA)
        (BaseVanillaEdpOozieEngine.get_possible_job_config.
            assert_called_once_with(edp.JOB_TYPE_JAVA))
        self.assertEqual(expected_config, actual_config)

    @mock.patch(
        'sahara_plugin_vanilla.plugins.vanilla.confighints_helper.'
        'get_possible_mapreduce_config_from',
        return_value={})
    def test_get_possible_job_config_mapreduce(
            self, get_possible_mapreduce_config_from):
        expected_config = {'job_config': {}}
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_MAPREDUCE)
        get_possible_mapreduce_config_from.assert_called_once_with(
            'plugins/vanilla/v2_7_5/resources/mapred-default.xml')
        self.assertEqual(expected_config, actual_config)

    @mock.patch(
        'sahara_plugin_vanilla.plugins.vanilla.confighints_helper.'
        'get_possible_mapreduce_config_from',
        return_value={})
    def test_get_possible_job_config_mapreduce_streaming(
            self, get_possible_mapreduce_config_from):
        expected_config = {'job_config': {}}
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_MAPREDUCE_STREAMING)
        get_possible_mapreduce_config_from.assert_called_once_with(
            'plugins/vanilla/v2_7_5/resources/mapred-default.xml')
        self.assertEqual(expected_config, actual_config)

    @mock.patch(
        'sahara_plugin_vanilla.plugins.vanilla.confighints_helper.'
        'get_possible_pig_config_from',
        return_value={})
    def test_get_possible_job_config_pig(
            self, get_possible_pig_config_from):
        expected_config = {'job_config': {}}
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_PIG)
        get_possible_pig_config_from.assert_called_once_with(
            'plugins/vanilla/v2_7_5/resources/mapred-default.xml')
        self.assertEqual(expected_config, actual_config)

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.hadoop2.edp_engine.'
                'EdpOozieEngine')
    def test_get_possible_job_config_shell(self, BaseVanillaEdpOozieEngine):
        expected_config = {'job_config': {}}
        BaseVanillaEdpOozieEngine.get_possible_job_config.return_value = (
            expected_config)
        actual_config = edp_engine.EdpOozieEngine.get_possible_job_config(
            edp.JOB_TYPE_SHELL)
        (BaseVanillaEdpOozieEngine.get_possible_job_config.
            assert_called_once_with(edp.JOB_TYPE_SHELL))
        self.assertEqual(expected_config, actual_config)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import edp
from sahara.plugins import exceptions as ex
from sahara.plugins import utils as u
from sahara_plugin_vanilla.plugins.vanilla import utils as vu


class EdpOozieEngine(edp.PluginsOozieJobEngine):
    def get_hdfs_user(self):
        return 'hadoop'

    def get_name_node_uri(self, cluster):
        return cluster['info']['HDFS']['NameNode']

    def get_oozie_server_uri(self, cluster):
        return cluster['info']['JobFlow']['Oozie'] + "/oozie/"

    def get_oozie_server(self, cluster):
        return vu.get_oozie(cluster)

    def validate_job_execution(self, cluster, job, data):
        oo_count = u.get_instances_count(cluster, 'oozie')
        if oo_count != 1:
            raise ex.InvalidComponentCountException('oozie', '1', oo_count)

        super(EdpOozieEngine, self).validate_job_execution(cluster, job, data)



# Copyright (c) 2014 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
import six

from sahara.plugins import exceptions as ex
from sahara.plugins import provisioning as p
from sahara.plugins import utils
from sahara_plugin_vanilla.i18n import _

CONF = cfg.CONF
CONF.import_opt("enable_data_locality", "sahara.topology.topology_helper")

HIDDEN_CONFS = [
    'dfs.hosts',
    'dfs.hosts.exclude',
    'dfs.namenode.data.dir',
    'dfs.namenode.name.dir',
    'fs.default.name',
    'fs.defaultFS',
    'fs.swift.impl',
    'hadoop.proxyuser.hadoop.groups',
    'hadoop.proxyuser.hadoop.hosts',
    'mapreduce.framework.name',
    'mapreduce.jobhistory.address',
    'mapreduce.jobhistory.done.dir',
    'mapreduce.jobhistory.intermediate-done-dir',
    'mapreduce.jobhistory.webapp.address',
    'yarn.nodemanager.aux-services',
    'yarn.resourcemanager.address',
    'yarn.resourcemanager.admin.address',
    'yarn.resourcemanager.hostname',
    'yarn.resourcemanager.nodes.exclude-path',
    'yarn.resourcemanager.nodes.include-path',
    'yarn.resourcemanager.resource-tracker.address',
    'yarn.resourcemanager.scheduler.address',
    'yarn.resourcemanager.webapp.address'
]

CLUSTER_WIDE_CONFS = [
    'dfs.blocksize', 'dfs.namenode.replication.min', 'dfs.permissions.enabled',
    'dfs.replication', 'dfs.replication.max', 'io.compression.codecs',
    'io.file.buffer.size', 'mapreduce.job.counters.max',
    'mapreduce.map.output.compress.codec',
    'mapreduce.output.fileoutputformat.compress.codec',
    'mapreduce.output.fileoutputformat.compress.type',
    'mapredude.map.output.compress',
    'mapredude.output.fileoutputformat.compress'
]

PRIORITY_1_CONFS = [
    'dfs.datanode.du.reserved',
    'dfs.datanode.failed.volumes.tolerated',
    'dfs.datanode.handler.count',
    'dfs.datanode.max.transfer.threads',
    'dfs.namenode.handler.count',
    'mapred.child.java.opts',
    'mapred.jobtracker.maxtasks.per.job',
    'mapreduce.jobtracker.handler.count',
    'mapreduce.map.java.opts',
    'mapreduce.reduce.java.opts',
    'mapreduce.task.io.sort.mb',
    'mapreduce.tasktracker.map.tasks.maximum',
    'mapreduce.tasktracker.reduce.tasks.maximum',
    'yarn.nodemanager.resource.cpu-vcores',
    'yarn.nodemanager.resource.memory-mb',
    'yarn.scheduler.maximum-allocation-mb',
    'yarn.scheduler.maximum-allocation-vcores',
    'yarn.scheduler.minimum-allocation-mb',
    'yarn.scheduler.minimum-allocation-vcores'
]

SPARK_CONFS = {
    'Spark': {
        "OPTIONS": [
            {
                'name': 'Spark home',
                'description': 'The location of the spark installation'
                ' (default: /opt/spark)',
                'default': '/opt/spark',
                'priority': 2,
            },
            {
                'name': 'Minimum cleanup seconds',
                'description': 'Job data will never be purged before this'
                ' amount of time elapses (default: 86400 = 1 day)',
                'default': '86400',
                'priority': 2,
            },
            {
                'name': 'Maximum cleanup seconds',
                'description': 'Job data will always be purged after this'
                ' amount of time elapses (default: 1209600 = 14 days)',
                'default': '1209600',
                'priority': 2,
            },
            {
                'name': 'Minimum cleanup megabytes',
                'description': 'No job data will be purged unless the total'
                ' job data exceeds this size (default: 4096 = 4GB)',
                'default': '4096',
                'priority': 2,
            },
        ]
    }
}

ZOOKEEPER_CONFS = {
    "ZooKeeper": {
        "OPTIONS": [
            {
                'name': 'tickTime',
                'description': 'The number of milliseconds of each tick',
                'default': 2000,
                'priority': 2,
            },
            {
                'name': 'initLimit',
                'description': 'The number of ticks that the initial'
                ' synchronization phase can take',
                'default': 10,
                'priority': 2,
            },
            {
                'name': 'syncLimit',
                'description': 'The number of ticks that can pass between'
                ' sending a request and getting an acknowledgement',
                'default': 5,
                'priority': 2,
            },
        ]
    }
}

# for now we have not so many cluster-wide configs
# lets consider all of them having high priority
PRIORITY_1_CONFS += CLUSTER_WIDE_CONFS


def init_xml_configs(xml_confs):
    configs = []
    for service, config_lists in six.iteritems(xml_confs):
        for config_list in config_lists:
            for config in config_list:
                if config['name'] not in HIDDEN_CONFS:
                    cfg = p.Config(config['name'], service, "node",
                                   is_optional=True, config_type="string",
                                   default_value=str(config['value']),
                                   description=config['description'])
                    if cfg.default_value in ["true", "false"]:
                        cfg.config_type = "bool"
                        cfg.default_value = (cfg.default_value == 'true')
                    elif utils.is_int(cfg.default_value):
                        cfg.config_type = "int"
                        cfg.default_value = int(cfg.default_value)
                    if config['name'] in CLUSTER_WIDE_CONFS:
                        cfg.scope = 'cluster'
                    if config['name'] in PRIORITY_1_CONFS:
                        cfg.priority = 1
                    configs.append(cfg)

    return configs


ENABLE_SWIFT = p.Config('Enable Swift', 'general', 'cluster',
                        config_type="bool", priority=1,
                        default_value=True, is_optional=False)

ENABLE_MYSQL = p.Config('Enable MySQL', 'general', 'cluster',
                        config_type="bool", priority=1,
                        default_value=True, is_optional=True)

ENABLE_DATA_LOCALITY = p.Config('Enable Data Locality', 'general', 'cluster',
                                config_type="bool", priority=1,
                                default_value=True, is_optional=True)


DATANODES_DECOMMISSIONING_TIMEOUT = p.Config(
    'DataNodes decommissioning timeout', 'general',
    'cluster', config_type='int', priority=1,
    default_value=3600 * 4, is_optional=True,
    description='Timeout for datanode decommissioning operation'
                ' during scaling, in seconds')


NODEMANAGERS_DECOMMISSIONING_TIMEOUT = p.Config(
    'NodeManagers decommissioning timeout', 'general',
    'cluster', config_type='int', priority=1,
    default_value=300, is_optional=True,
    description='Timeout for NodeManager decommissioning operation'
                ' during scaling, in seconds')


DATANODES_STARTUP_TIMEOUT = p.Config(
    'DataNodes startup timeout', 'general', 'cluster', config_type='int',
    priority=1, default_value=10800, is_optional=True,
    description='Timeout for DataNodes startup, in seconds')


def init_env_configs(env_confs):
    configs = []
    for service, config_items in six.iteritems(env_confs):
        for name, value in six.iteritems(config_items):
            configs.append(p.Config(name, service, "node",
                                    default_value=value, priority=1,
                                    config_type="int"))

    return configs


def _init_general_configs():
    configs = [ENABLE_SWIFT, ENABLE_MYSQL, DATANODES_STARTUP_TIMEOUT,
               DATANODES_DECOMMISSIONING_TIMEOUT,
               NODEMANAGERS_DECOMMISSIONING_TIMEOUT]
    if CONF.enable_data_locality:
        configs.append(ENABLE_DATA_LOCALITY)
    return configs


PLUGIN_GENERAL_CONFIGS = _init_general_configs()


def get_config_value(pctx, service, name, cluster=None):
    if cluster:
        for ng in cluster.node_groups:
            cl_param = ng.configuration().get(service, {}).get(name)
            if cl_param is not None:
                return cl_param

    for c in pctx['all_confs']:
        if c.applicable_target == service and c.name == name:
            return c.default_value

    raise ex.PluginNotFoundException(
        {"name": name, "service": service},
        _("Unable to get parameter '%(name)s' from service %(service)s"))


def is_swift_enabled(pctx, cluster):
    return get_config_value(pctx, ENABLE_SWIFT.applicable_target,
                            ENABLE_SWIFT.name, cluster)


def is_mysql_enabled(pctx, cluster):
    return get_config_value(
        pctx, ENABLE_MYSQL.applicable_target, ENABLE_MYSQL.name, cluster)


def is_data_locality_enabled(pctx, cluster):
    if not CONF.enable_data_locality:
        return False
    return get_config_value(pctx, ENABLE_DATA_LOCALITY.applicable_target,
                            ENABLE_DATA_LOCALITY.name, cluster)


def generate_spark_env_configs(cluster):
    configs = []

    # point to the hadoop conf dir so that Spark can read things
    # like the swift configuration without having to copy core-site
    # to /opt/spark/conf
    HADOOP_CONF_DIR = '/opt/hadoop/etc/hadoop'
    configs.append('HADOOP_CONF_DIR=' + HADOOP_CONF_DIR)

    # Hadoop and YARN configs there are in one folder
    configs.append('YARN_CONF_DIR=' + HADOOP_CONF_DIR)

    return '\n'.join(configs)


def generate_spark_executor_classpath(cluster):
    cp = utils.get_config_value_or_default(
        "Spark", "Executor extra classpath", cluster)
    if cp:
        return "spark.executor.extraClassPath " + cp
    return "\n"


def generate_job_cleanup_config(cluster):
    args = {
        'minimum_cleanup_megabytes': utils.get_config_value_or_default(
            "Spark", "Minimum cleanup megabytes", cluster),
        'minimum_cleanup_seconds': utils.get_config_value_or_default(
            "Spark", "Minimum cleanup seconds", cluster),
        'maximum_cleanup_seconds': utils.get_config_value_or_default(
            "Spark", "Maximum cleanup seconds", cluster)
    }
    job_conf = {'valid': (args['maximum_cleanup_seconds'] > 0 and
                          (args['minimum_cleanup_megabytes'] > 0
                           and args['minimum_cleanup_seconds'] > 0))}
    if job_conf['valid']:
        job_conf['cron'] = utils.get_file_text(
            'plugins/vanilla/hadoop2/resources/spark-cleanup.cron',
            'sahara_plugin_vanilla'),
        job_cleanup_script = utils.get_file_text(
            'plugins/vanilla/hadoop2/resources/tmp-cleanup.sh.template',
            'sahara_plugin_vanilla')
        job_conf['script'] = job_cleanup_script.format(**args)
    return job_conf


def get_spark_home(cluster):
    return utils.get_config_value_or_default("Spark", "Spark home", cluster)


def generate_zk_basic_config(cluster):
    args = {
        'ticktime': utils.get_config_value_or_default(
            "ZooKeeper", "tickTime", cluster),
        'initlimit': utils.get_config_value_or_default(
            "ZooKeeper", "initLimit", cluster),
        'synclimit': utils.get_config_value_or_default(
            "ZooKeeper", "syncLimit", cluster)
    }
    zoo_cfg = utils.get_file_text(
        'plugins/vanilla/hadoop2/resources/zoo_sample.cfg',
        'sahara_plugin_vanilla')
    return zoo_cfg.format(**args)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import recommendations_utils as ru


def recommend_configs(cluster, plugin_configs, scaling):
    yarn_configs = [
        'yarn.nodemanager.resource.memory-mb',
        'yarn.scheduler.minimum-allocation-mb',
        'yarn.scheduler.maximum-allocation-mb',
        'yarn.nodemanager.vmem-check-enabled',
    ]
    mapred_configs = [
        'yarn.app.mapreduce.am.resource.mb',
        'yarn.app.mapreduce.am.command-opts',
        'mapreduce.map.memory.mb',
        'mapreduce.reduce.memory.mb',
        'mapreduce.map.java.opts',
        'mapreduce.reduce.java.opts',
        'mapreduce.task.io.sort.mb',
    ]
    configs_to_configure = {
        'cluster_configs': {
            'dfs.replication': ('HDFS', 'dfs.replication')
        },
        'node_configs': {
        }
    }
    for mapr in mapred_configs:
        configs_to_configure['node_configs'][mapr] = ("MapReduce", mapr)
    for yarn in yarn_configs:
        configs_to_configure['node_configs'][yarn] = ('YARN', yarn)
    provider = ru.HadoopAutoConfigsProvider(
        configs_to_configure, plugin_configs, cluster, scaling)
    provider.apply_recommended_configs()



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara.plugins import swift_helper
from sahara.plugins import utils
from sahara_plugin_vanilla.i18n import _
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config_helper
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import run_scripts as run
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import utils as pu
from sahara_plugin_vanilla.plugins.vanilla import utils as vu

HADOOP_CONF_DIR = config.HADOOP_CONF_DIR


def scale_cluster(pctx, cluster, instances):
    config.configure_instances(pctx, instances)
    _update_include_files(cluster)
    run.refresh_hadoop_nodes(cluster)
    rm = vu.get_resourcemanager(cluster)
    if rm:
        run.refresh_yarn_nodes(cluster)

    config.configure_topology_data(pctx, cluster)
    run.start_dn_nm_processes(instances)
    swift_helper.install_ssl_certs(instances)
    config.configure_zookeeper(cluster)
    run.refresh_zk_servers(cluster)


def _get_instances_with_service(instances, service):
    return [instance for instance in instances
            if service in instance.node_group.node_processes]


@utils.event_wrapper(
    True, step=_("Update include files"), param=('cluster', 0))
def _update_include_files(cluster, dec_instances=None):
    dec_instances = dec_instances or []
    dec_instances_ids = [instance.id for instance in dec_instances]

    instances = utils.get_instances(cluster)

    inst_filter = lambda inst: inst.id not in dec_instances_ids

    datanodes = filter(inst_filter, vu.get_datanodes(cluster))
    nodemanagers = filter(inst_filter, vu.get_nodemanagers(cluster))
    dn_hosts = utils.generate_fqdn_host_names(datanodes)
    nm_hosts = utils.generate_fqdn_host_names(nodemanagers)
    for instance in instances:
        with instance.remote() as r:
            r.execute_command(
                'sudo su - -c "echo \'%s\' > %s/dn-include" hadoop' % (
                    dn_hosts, HADOOP_CONF_DIR))
            r.execute_command(
                'sudo su - -c "echo \'%s\' > %s/nm-include" hadoop' % (
                    nm_hosts, HADOOP_CONF_DIR))


def decommission_nodes(pctx, cluster, instances):
    datanodes = _get_instances_with_service(instances, 'datanode')
    nodemanagers = _get_instances_with_service(instances, 'nodemanager')
    _update_exclude_files(cluster, instances)

    run.refresh_hadoop_nodes(cluster)
    rm = vu.get_resourcemanager(cluster)
    if rm:
        run.refresh_yarn_nodes(cluster)

    _check_nodemanagers_decommission(cluster, nodemanagers)
    _check_datanodes_decommission(cluster, datanodes)

    _update_include_files(cluster, instances)
    _clear_exclude_files(cluster)
    run.refresh_hadoop_nodes(cluster)

    config.configure_topology_data(pctx, cluster)
    config.configure_zookeeper(cluster, instances)
    # TODO(shuyingya):should invent a way to lastly restart the leader node
    run.refresh_zk_servers(cluster, instances)


def _update_exclude_files(cluster, instances):
    datanodes = _get_instances_with_service(instances, 'datanode')
    nodemanagers = _get_instances_with_service(instances, 'nodemanager')
    dn_hosts = utils.generate_fqdn_host_names(datanodes)
    nm_hosts = utils.generate_fqdn_host_names(nodemanagers)
    for instance in utils.get_instances(cluster):
        with instance.remote() as r:
            r.execute_command(
                'sudo su - -c "echo \'%s\' > %s/dn-exclude" hadoop' % (
                    dn_hosts, HADOOP_CONF_DIR))
            r.execute_command(
                'sudo su - -c "echo \'%s\' > %s/nm-exclude" hadoop' % (
                    nm_hosts, HADOOP_CONF_DIR))


def _clear_exclude_files(cluster):
    for instance in utils.get_instances(cluster):
        with instance.remote() as r:
            r.execute_command(
                'sudo su - -c "echo > %s/dn-exclude" hadoop' % HADOOP_CONF_DIR)
            r.execute_command(
                'sudo su - -c "echo > %s/nm-exclude" hadoop' % HADOOP_CONF_DIR)


def is_decommissioned(cluster, check_func, instances):
    statuses = check_func(cluster)
    for instance in instances:
        if statuses[instance.fqdn()] != 'decommissioned':
            return False
    return True


def _check_decommission(cluster, instances, check_func, option):
    utils.plugin_option_poll(
        cluster, is_decommissioned, option, _("Wait for decommissioning"),
        5, {'cluster': cluster, 'check_func': check_func,
            'instances': instances})


@utils.event_wrapper(
    True, step=_("Decommission %s") % "NodeManagers", param=('cluster', 0))
def _check_nodemanagers_decommission(cluster, instances):
    _check_decommission(cluster, instances, pu.get_nodemanagers_status,
                        config_helper.NODEMANAGERS_DECOMMISSIONING_TIMEOUT)


@utils.event_wrapper(
    True, step=_("Decommission %s") % "DataNodes", param=('cluster', 0))
def _check_datanodes_decommission(cluster, instances):
    _check_decommission(cluster, instances, pu.get_datanodes_status,
                        config_helper.DATANODES_DECOMMISSIONING_TIMEOUT)



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import provisioning as p
from sahara_plugin_vanilla.plugins.vanilla.v2_8_2 import config_helper
from sahara_plugin_vanilla.tests.unit import base


class TestConfigHelper(base.SaharaTestCase):

    plugin_path = 'sahara_plugin_vanilla.plugins.vanilla.v2_8_2.'
    plugin_hadoop_path = 'sahara_plugin_vanilla.plugins.vanilla.hadoop2.'

    def setUp(self):
        super(TestConfigHelper, self).setUp()

    @mock.patch(plugin_hadoop_path + 'config_helper.PLUGIN_GENERAL_CONFIGS')
    @mock.patch(plugin_path + 'config_helper.PLUGIN_ENV_CONFIGS')
    @mock.patch(plugin_path + 'config_helper.PLUGIN_XML_CONFIGS')
    @mock.patch(plugin_path + 'config_helper._get_spark_configs')
    @mock.patch(plugin_path + 'config_helper._get_zookeeper_configs')
    def test_init_all_configs(self,
                              _get_zk_configs,
                              _get_spark_configs,
                              PLUGIN_XML_CONFIGS,
                              PLUGIN_ENV_CONFIGS,
                              PLUGIN_GENERAL_CONFIGS):
        configs = []
        configs.extend(PLUGIN_XML_CONFIGS)
        configs.extend(PLUGIN_ENV_CONFIGS)
        configs.extend(PLUGIN_GENERAL_CONFIGS)
        configs.extend(_get_spark_configs())
        configs.extend(_get_zk_configs())
        init_configs = config_helper._init_all_configs()
        self.assertEqual(init_configs, configs)

    def test_get_spark_opt_default(self):
        opt_name = 'Executor extra classpath'
        _default_executor_classpath = ":".join(
            ['/opt/hadoop/share/hadoop/tools/lib/hadoop-openstack-2.8.2.jar'])
        default = config_helper._get_spark_opt_default(opt_name)
        self.assertEqual(default, _default_executor_classpath)

    def test_get_spark_configs(self):
        spark_configs = config_helper._get_spark_configs()
        for i in spark_configs:
            self.assertIsInstance(i, p.Config)

    def test_get_plugin_configs(self):
        self.assertEqual(config_helper.get_plugin_configs(),
                         config_helper.PLUGIN_CONFIGS)

    def test_get_xml_configs(self):
        self.assertEqual(config_helper.get_xml_configs(),
                         config_helper.PLUGIN_XML_CONFIGS)

    def test_get_env_configs(self):
        self.assertEqual(config_helper.get_env_configs(),
                         config_helper.ENV_CONFS)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import base as pb
from sahara.plugins import conductor
from sahara.plugins import context
from sahara.plugins import edp
from sahara_plugin_vanilla.tests.unit import base


class VanillaPluginTest(base.SaharaWithDbTestCase):
    def setUp(self):
        super(VanillaPluginTest, self).setUp()
        self.override_config("plugins", ["vanilla"])
        pb.setup_plugins()

    @mock.patch('sahara.plugins.edp.create_dir_hadoop2')
    def test_edp_calls_hadoop2_create_dir(self, create_dir):
        for version in ['2.7.1']:
            cluster_dict = {
                'name': 'cluster' + version.replace('.', '_'),
                'plugin_name': 'vanilla',
                'hadoop_version': version,
                'default_image_id': 'image'}

            cluster = conductor.cluster_create(context.ctx(), cluster_dict)
            plugin = pb.PLUGINS.get_plugin(cluster.plugin_name)
            create_dir.reset_mock()
            plugin.get_edp_engine(cluster, edp.JOB_TYPE_PIG).create_hdfs_dir(
                mock.Mock(), '/tmp')
            self.assertEqual(1, create_dir.call_count)



# Copyright (c) 2017 EasyStack Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara_plugin_vanilla.i18n import _
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import config_helper
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import scaling
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import utils as pu
from sahara_plugin_vanilla.tests.unit import base


class ScalingTest(base.SaharaTestCase):

    PLUGINS_PATH = 'sahara_plugin_vanilla.plugins.vanilla.hadoop2.'

    def setUp(self):
        super(ScalingTest, self).setUp()
        self.cluster = mock.Mock()
        self.instances = mock.Mock()
        self.r = mock.Mock()
        self.r.execute_command = mock.Mock()
        self.instance = mock.Mock()
        self.instance.remote.return_value.__enter__ = mock.Mock(
            return_value=self.r)
        self.instance.remote.return_value.__exit__ = mock.Mock()

    @mock.patch('sahara.plugins.swift_helper.install_ssl_certs')
    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.'
                'get_resourcemanager')
    @mock.patch(PLUGINS_PATH + 'run_scripts.refresh_zk_servers')
    @mock.patch(PLUGINS_PATH + 'config.configure_zookeeper')
    @mock.patch(PLUGINS_PATH + 'run_scripts.start_dn_nm_processes')
    @mock.patch(PLUGINS_PATH + 'run_scripts.refresh_yarn_nodes')
    @mock.patch(PLUGINS_PATH + 'run_scripts.refresh_hadoop_nodes')
    @mock.patch(PLUGINS_PATH + 'scaling._update_include_files')
    @mock.patch(PLUGINS_PATH + 'config.configure_topology_data')
    @mock.patch(PLUGINS_PATH + 'config.configure_instances')
    def test_scale_cluster(self, configure_instances,
                           configure_topology_data,
                           _update_include_files,
                           refresh_hadoop_nodes,
                           refresh_yarn_nodes,
                           start_dn_nm_processes,
                           configure_zk, refresh_zk,
                           get_resourcemanager,
                           install_ssl_certs):
        get_resourcemanager.return_value = 'node1'
        pctx = mock.Mock()
        scaling.scale_cluster(pctx, self.cluster, self.instances)
        configure_instances.assert_called_once_with(pctx, self.instances)
        _update_include_files.assert_called_once_with(self.cluster)
        refresh_hadoop_nodes.assert_called_once_with(self.cluster)
        get_resourcemanager.assert_called_once_with(self.cluster)
        refresh_yarn_nodes.assert_called_once_with(self.cluster)
        configure_topology_data.assert_called_once_with(pctx, self.cluster)
        start_dn_nm_processes.assert_called_once_with(self.instances)
        install_ssl_certs.assert_called_once_with(self.instances)
        configure_topology_data.assert_called_once_with(pctx, self.cluster)
        configure_zk.assert_called_once_with(self.cluster)
        refresh_zk.assert_called_once_with(self.cluster)

    def test_get_instances_with_service(self):
        ins_1 = mock.Mock()
        ins_1.node_group.node_processes = ['nodename']
        ins_2 = mock.Mock()
        ins_2.node_group.node_processes = ['nodedata']
        instances = [ins_1, ins_2]
        service = 'nodename'
        ret = scaling._get_instances_with_service(instances, service)
        self.assertEqual(ret, [ins_1])

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.get_nodemanagers')
    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.get_datanodes')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    @mock.patch('sahara.plugins.utils.generate_fqdn_host_names')
    @mock.patch('sahara.plugins.utils.get_instances')
    def test_update_include_files(self, get_instances,
                                  generate_fqdn_host_names,
                                  add_provisioning_step,
                                  check_cluster_exists,
                                  get_datanodes, get_nodemanagers):
        DIR = scaling.HADOOP_CONF_DIR
        host = '1.2.3.4'
        ins_1 = mock.Mock()
        ins_1.id = 'instance_1'
        ins_2 = mock.Mock()
        ins_2.id = 'instance_2'
        ins_3 = mock.Mock()
        ins_3.id = 'instance_3'
        ins_4 = mock.Mock()
        ins_4.id = 'instance_4'
        dec_instances = [ins_1, ins_2]
        get_instances.return_value = [self.instance]
        get_datanodes.return_value = [ins_3]
        get_nodemanagers.return_value = [ins_4]
        generate_fqdn_host_names.return_value = host
        scaling._update_include_files(self.cluster, dec_instances)
        get_instances.assert_called_once_with(self.cluster)
        get_datanodes.assert_called_once_with(self.cluster)
        get_nodemanagers.assert_called_once_with(self.cluster)
        count = generate_fqdn_host_names.call_count
        self.assertEqual(count, 2)
        command_calls = [mock.call(
            'sudo su - -c "echo \'%s\' > %s/dn-include" hadoop' % (
                host, DIR)), mock.call(
            'sudo su - -c "echo \'%s\' > %s/nm-include" hadoop' % (
                host, DIR))]
        self.r.execute_command.assert_has_calls(command_calls, any_order=True)

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.'
                'get_resourcemanager')
    @mock.patch(PLUGINS_PATH + 'run_scripts.refresh_zk_servers')
    @mock.patch(PLUGINS_PATH + 'config.configure_zookeeper')
    @mock.patch(PLUGINS_PATH + 'config.configure_topology_data')
    @mock.patch(PLUGINS_PATH + 'run_scripts.refresh_yarn_nodes')
    @mock.patch(PLUGINS_PATH + 'run_scripts.refresh_hadoop_nodes')
    @mock.patch(PLUGINS_PATH + 'scaling._update_exclude_files')
    @mock.patch(PLUGINS_PATH + 'scaling._clear_exclude_files')
    @mock.patch(PLUGINS_PATH + 'scaling._update_include_files')
    @mock.patch(PLUGINS_PATH + 'scaling._check_datanodes_decommission')
    @mock.patch(PLUGINS_PATH + 'scaling._check_nodemanagers_decommission')
    @mock.patch(PLUGINS_PATH + 'scaling._get_instances_with_service')
    def test_decommission_nodes(self, _get_instances_with_service,
                                _check_nodemanagers_decommission,
                                _check_datanodes_decommission,
                                _update_include_files, _clear_exclude_files,
                                _update_exclude_files, refresh_hadoop_nodes,
                                refresh_yarn_nodes, configure_topology_data,
                                configure_zk, refresh_zk,
                                get_resourcemanager):
        data = 'test_data'
        _get_instances_with_service.return_value = data
        get_resourcemanager.return_value = 'node1'
        pctx = mock.Mock()
        scaling.decommission_nodes(pctx, self.cluster, self.instances)
        get_instances_count = _get_instances_with_service.call_count
        self.assertEqual(get_instances_count, 2)
        _update_exclude_files.assert_called_once_with(self.cluster,
                                                      self.instances)
        refresh_count = refresh_hadoop_nodes.call_count
        self.assertEqual(refresh_count, 2)
        get_resourcemanager.assert_called_once_with(self.cluster)
        refresh_yarn_nodes.assert_called_once_with(self.cluster)
        _check_nodemanagers_decommission.assert_called_once_with(
            self.cluster, data)
        _check_datanodes_decommission.assert_called_once_with(
            self.cluster, data)
        _update_include_files.assert_called_once_with(self.cluster,
                                                      self.instances)
        _clear_exclude_files.assert_called_once_with(self.cluster)
        configure_topology_data.assert_called_once_with(pctx, self.cluster)
        configure_zk.assert_called_once_with(self.cluster, self.instances)
        refresh_zk.assert_called_once_with(self.cluster, self.instances)

    @mock.patch(PLUGINS_PATH + 'scaling._get_instances_with_service')
    @mock.patch('sahara.plugins.utils.generate_fqdn_host_names')
    @mock.patch('sahara.plugins.utils.get_instances')
    def test_update_exclude_files(self, get_instances,
                                  generate_fqdn_host_names,
                                  get_instances_with_service):
        node = mock.Mock()
        get_instances_with_service.return_value = node
        host = '1.2.3.4'
        generate_fqdn_host_names.return_value = host
        get_instances.return_value = [self.instance]
        scaling._update_exclude_files(self.cluster, self.instances)
        service_calls = [mock.call(self.instances, 'datanode'),
                         mock.call(self.instances, 'nodemanager')]
        get_instances_with_service.assert_has_calls(service_calls,
                                                    any_order=True)
        self.assertEqual(generate_fqdn_host_names.call_count, 2)
        get_instances.assert_called_once_with(self.cluster)
        DIR = scaling.HADOOP_CONF_DIR
        command_calls = [mock.call(
            'sudo su - -c "echo \'%s\' > %s/dn-exclude" hadoop' % (
                host, DIR)), mock.call(
            'sudo su - -c "echo \'%s\' > %s/nm-exclude" hadoop' % (
                host, DIR))]
        self.r.execute_command.assert_has_calls(command_calls, any_order=True)

    @mock.patch('sahara.plugins.utils.get_instances')
    def test_clear_exclude_files(self, get_instances):
        get_instances.return_value = [self.instance]
        scaling._clear_exclude_files(self.cluster)
        get_instances.assert_called_once_with(self.cluster)
        DIR = scaling.HADOOP_CONF_DIR
        calls = [mock.call('sudo su - -c "echo > %s/dn-exclude" hadoop' %
                           DIR),
                 mock.call('sudo su - -c "echo > %s/nm-exclude" hadoop' %
                           DIR)]
        self.r.execute_command.assert_has_calls(calls, any_order=True)

    def test_is_decommissioned(self):
        def check_func(cluster):
            statuses = {'status': cluster}
            return statuses
        ins = mock.Mock()
        ins.fqdn.return_value = 'status'
        instances = [ins]
        cluster = 'decommissioned'
        ret = scaling.is_decommissioned(cluster, check_func, instances)
        self.assertEqual(ret, True)

        cluster = 'active'
        ret = scaling.is_decommissioned(cluster, check_func, instances)
        self.assertEqual(ret, False)

    @mock.patch('sahara.plugins.utils.plugin_option_poll')
    def test_check_decommission(self, plugin_option_poll):
        check_func = mock.Mock()
        option = mock.Mock()
        is_dec = scaling.is_decommissioned
        mess = _("Wait for decommissioning")
        sample_dict = {'cluster': self.cluster,
                       'check_func': check_func,
                       'instances': self.instances}
        scaling._check_decommission(self.cluster, self.instances,
                                    check_func, option)
        plugin_option_poll.assert_called_once_with(self.cluster, is_dec,
                                                   option, mess, 5,
                                                   sample_dict)

    @mock.patch(PLUGINS_PATH + 'scaling._check_decommission')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    def test_check_nodemanagers_decommission(self, add_provisioning_step,
                                             check_cluster_exists,
                                             _check_decommission):
        timeout = config_helper.NODEMANAGERS_DECOMMISSIONING_TIMEOUT
        status = pu.get_nodemanagers_status
        scaling._check_nodemanagers_decommission(self.cluster, self.instances)
        _check_decommission.assert_called_once_with(self.cluster,
                                                    self.instances,
                                                    status, timeout)

    @mock.patch(PLUGINS_PATH + 'scaling._check_decommission')
    @mock.patch('sahara.plugins.utils.check_cluster_exists')
    @mock.patch('sahara.plugins.utils.add_provisioning_step')
    def test_check_datanodes_decommission(self, add_provisioning_step,
                                          check_cluster_exists,
                                          _check_decommission):
        timeout = config_helper.DATANODES_DECOMMISSIONING_TIMEOUT
        status = pu.get_datanodes_status
        scaling._check_datanodes_decommission(self.cluster, self.instances)
        _check_decommission.assert_called_once_with(self.cluster,
                                                    self.instances,
                                                    status, timeout)



# Copyright (c) 2016 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from castellan.common.objects import passphrase
from castellan import key_manager
from oslo_log import log as logging


from sahara.plugins import conductor
from sahara.plugins import context
from sahara.plugins import utils

LOG = logging.getLogger(__name__)


def _provision_key(instance, keypair):
    def append_to(remote, file, *args, **kwargs):
        kwargs['run_as_root'] = True
        path = "/home/hadoop/.ssh/%s" % file
        remote.append_to_file(path, *args, **kwargs)
    public, private = keypair['public'], keypair['private']
    folder = '/home/hadoop/.ssh'
    with context.set_current_instance_id(instance_id=instance.instance_id):
        with instance.remote() as r:
            r.execute_command('sudo mkdir -p %s' % folder)
            append_to(r, 'authorized_keys', public)
            append_to(r, 'id_rsa', private)
            append_to(r, 'id_rsa.pub', public)
            r.execute_command('sudo chown -R hadoop %s' % folder)
            r.execute_command("sudo chmod 600 %s/id_rsa" % folder)
        LOG.debug("Passwordless ssh enabled")


def _get_secret(secret):
    key = key_manager.API().get(context.current(), secret)
    return key.get_encoded()


def _store_secret(secret):
    key = passphrase.Passphrase(secret)
    password = key_manager.API().store(context.current(), key)
    return password


def _remove_secret(secret):
    key_manager.API().delete(context.current(), secret)


def provision_keypairs(cluster, instances=None):
    extra = cluster.extra.to_dict() if cluster.extra else {}
    # use same keypair for scaling
    keypair = extra.get('vanilla_keypair')
    if not instances:
        instances = utils.get_instances(cluster)
    else:
        # scaling
        if not keypair:
            # cluster created before mitaka, skipping provisioning
            return
    if not keypair:
        private, public = utils.generate_key_pair()
        keypair = {'public': public, 'private': private}
        extra['vanilla_keypair'] = keypair
        extra['vanilla_keypair']['private'] = _store_secret(
            keypair['private'])
        conductor.cluster_update(context.ctx(), cluster, {'extra': extra})
    else:
        keypair['private'] = _get_secret(keypair['private'])
    with context.PluginsThreadGroup() as tg:
        for instance in instances:
            tg.spawn(
                'provision-key-%s' % instance.instance_name,
                _provision_key, instance, keypair)


def drop_key(cluster):
    extra = cluster.extra.to_dict() if cluster.extra else {}
    keypair = extra.get('vanilla_keypair')
    if keypair:
        _remove_secret(keypair['private'])



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.plugins import utils
from sahara_plugin_vanilla.plugins.vanilla.hadoop2 import utils as u
from sahara_plugin_vanilla.tests.unit import base


class UtilsTestCase(base.SaharaTestCase):
    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.get_namenode')
    def test_datanodes_status(self, nn):
        report = utils.get_file_text(
            'tests/unit/plugins/vanilla/hadoop2/resources/dfs-report.txt',
            'sahara_plugin_vanilla')

        nn.return_value = self._get_instance(report)
        statuses = u.get_datanodes_status(None)

        expected = {
            'cluster-worker-001.novalocal': 'normal',
            'cluster-worker-002.novalocal': 'normal',
            'cluster-worker-003.novalocal': 'normal',
            'cluster-worker-004.novalocal': 'decommissioned'
        }

        self.assertEqual(expected, statuses)

    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils.'
                'get_resourcemanager')
    def test_nodemanagers_status(self, rm):
        report = utils.get_file_text(
            'tests/unit/plugins/vanilla/hadoop2/resources/yarn-report.txt',
            'sahara_plugin_vanilla')

        rm.return_value = self._get_instance(report)
        statuses = u.get_nodemanagers_status(None)

        expected = {
            'cluster-worker-001.novalocal': 'running',
            'cluster-worker-002.novalocal': 'running',
            'cluster-worker-003.novalocal': 'running',
            'cluster-worker-004.novalocal': 'decommissioned'
        }

        self.assertEqual(expected, statuses)

    def _get_instance(self, out):
        inst_remote = mock.MagicMock()
        inst_remote.execute_command.return_value = 0, out
        inst_remote.__enter__.return_value = inst_remote

        inst = mock.MagicMock()
        inst.remote.return_value = inst_remote

        return inst

    @mock.patch('sahara.plugins.conductor.cluster_get')
    @mock.patch('sahara.plugins.castellan_utils.get_secret')
    @mock.patch('sahara.plugins.castellan_utils.store_secret')
    @mock.patch('sahara_plugin_vanilla.plugins.vanilla.utils')
    @mock.patch('sahara.plugins.conductor.cluster_update')
    def test_oozie_password(self, cluster_update, vu,
                            store_secret, get_secret, conductor):
        cluster = mock.MagicMock()
        cluster.extra = mock.MagicMock()
        cluster.extra.to_dict.return_value = {"oozie_pass_id": "31415926"}

        conductor.return_value = cluster

        get_secret.return_value = "oozie_pass"
        result = u.get_oozie_password(cluster)

        get_secret.assert_called_once_with("31415926")
        vu.generate_random_password.assert_not_called()
        self.assertEqual('oozie_pass', result)

        cluster.extra.to_dict.return_value = {}

        store_secret.return_value = 'oozie_pass'
        result = u.get_oozie_password(cluster)
        self.assertEqual('oozie_pass', result)

    @mock.patch('sahara.plugins.castellan_utils.delete_secret')
    def test_delete_oozie_password(self, delete_secret):
        cluster = mock.MagicMock()
        cluster.extra.to_dict = mock.MagicMock()

        cluster.extra.to_dict.return_value = {}
        u.delete_oozie_password(cluster)
        delete_secret.assert_not_called()

        cluster.extra.to_dict.return_value = {"oozie_pass_id": "31415926"}
        u.delete_oozie_password(cluster)
        delete_secret.assert_called_once_with("31415926")

    @mock.patch('sahara.plugins.conductor.cluster_get')
    @mock.patch('sahara.plugins.castellan_utils.get_secret')
    @mock.patch('sahara.plugins.castellan_utils.store_secret')
    @mock.patch('sahara.plugins.conductor.cluster_update')
    def test_get_hive_password(self, cluster_update,
                               store_secret, get_secret, conductor):
        cluster = mock.MagicMock()
        cluster.extra.to_dict.return_value = {"hive_pass_id": "31415926"}

        conductor.return_value = cluster

        get_secret.return_value = "hive_pass"
        result = u.get_hive_password(cluster)

        get_secret.assert_called_once_with("31415926")
        self.assertEqual('hive_pass', result)

        cluster.extra.to_dict.return_value = {}

        store_secret.return_value = 'hive_pass'
        result = u.get_hive_password(cluster)
        self.assertEqual('hive_pass', result)

    @mock.patch('sahara.plugins.castellan_utils.delete_secret')
    def test_delete_hive_password(self, delete_secret):
        cluster = mock.MagicMock()

        cluster.extra.to_dict.return_value = {}
        u.delete_hive_password(cluster)
        delete_secret.assert_not_called()

        cluster.extra.to_dict.return_value = {"hive_pass_id": "31415926"}

        u.delete_hive_password(cluster)
        delete_secret.assert_called_once_with("31415926")
